# Datei zur Erstellung der Bibliotheksdateien

# Alle Variablen fuer Hauptpfade
OBJECT_FILE_PATH = $(OUTPUT_FOLDER)/
LIBRARY_FILE_PATH = $(OUTPUT_FOLDER)/

CXX_FLAGS = -fvisibility=hidden -Wno-deprecated-declarations -Wno-ignored-qualifiers

# Alle Variablen fuer Projektklassen
WORKSPACE_PATH = ../
PROJECT_PATH = ./

# Variable fuer gemeinsam verwendete Dateien.
SHARED_INCLUDE_PATH = $(WORKSPACE_PATH)shared/include/
SHARED_SRC_PATH = $(WORKSPACE_PATH)shared/src/
# Variable fuer 3rd libraries
THIRD_PATH = $(WORKSPACE_PATH)3rd/
# Variable fuer Boost-Pfad.
BOOST_PATH = $(THIRD_PATH)boost/

# Alle Variablen fuer Project
SOURCE_PATH = $(PROJECT_PATH)src/
SOURCE_FILES = \
$(PROJECT_PATH)src/datafox_library_iv.cpp \
$(PROJECT_PATH)src/library_version.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(PROJECT_PATH) -I$(PROJECT_PATH)include/ -I$(SHARED_INCLUDE_PATH) -I$(SHARED_INCLUDE_PATH)file/ -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer TINYXML
SOURCE_PATH = $(THIRD_PATH)tinyxml/
SOURCE_FILES += \
$(SOURCE_PATH)tinystr.cpp \
$(SOURCE_PATH)tinyxml.cpp \
$(SOURCE_PATH)tinyxmlerror.cpp \
$(SOURCE_PATH)tinyxmlparser.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer ZIP Utils
SOURCE_PATH = $(THIRD_PATH)zip_utils/

# Aus den Sourcen die benoetigten Dateien ermitteln.
SOURCE_FILES_CPP = $(notdir $(SOURCE_FILES))
SOURCE_FILES_O = $(addprefix $(OBJECT_FILE_PATH), $(SOURCE_FILES_CPP:.cpp=.o))
SOURCE_FILES_D = $(addprefix $(OBJECT_FILE_PATH), $(SOURCE_FILES_CPP:.cpp=.d))



# Alle Variablen fuer API
SOURCE_PATH = $(PROJECT_PATH)src/api/
SOURCE_FILES += \
$(SOURCE_PATH)df_access_control.cpp \
$(SOURCE_PATH)df_access_control_lists.cpp \
$(SOURCE_PATH)df_access_control_lists_v1.cpp \
$(SOURCE_PATH)df_active_mode.cpp \
$(SOURCE_PATH)df_api.cpp \
$(SOURCE_PATH)df_connection.cpp \
$(SOURCE_PATH)df_files.cpp \
$(SOURCE_PATH)df_fingerprint.cpp \
$(SOURCE_PATH)df_lists.cpp \
$(SOURCE_PATH)df_logging.cpp \
$(SOURCE_PATH)df_setup_lists.cpp \
$(SOURCE_PATH)df_setup_records.cpp \
$(SOURCE_PATH)df_system_records.cpp \
$(SOURCE_PATH)df_timeboy_lists.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(PROJECT_PATH)include/ -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Kommunikationsklassen
SOURCE_PATH = $(SHARED_SRC_PATH)channel/
SOURCE_FILES += \
$(SOURCE_PATH)channel_1_2_buffer.cpp \
$(SOURCE_PATH)channel_1_2.cpp \
$(SOURCE_PATH)channel_4_buffer.cpp \
$(SOURCE_PATH)channel_4.cpp \
$(SOURCE_PATH)channel_bridge.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Allgemeines
SOURCE_PATH = $(SHARED_SRC_PATH)common/
SOURCE_FILES += \
$(SOURCE_PATH)crc.cpp \
$(SOURCE_PATH)device_info.cpp \
$(SOURCE_PATH)df4_crypt.cpp \
$(SOURCE_PATH)dfcom_utils.cpp \
$(SOURCE_PATH)firmware_options.cpp \
$(SOURCE_PATH)hardware_info.cpp \
$(SOURCE_PATH)md5.cpp \
$(SOURCE_PATH)software_info.cpp \
$(SOURCE_PATH)software_version.cpp \
$(SOURCE_PATH)tcpip_info.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Kommunikationsklassen
SOURCE_PATH = $(SHARED_SRC_PATH)datafox/communication/
SOURCE_FILES += \
$(SOURCE_PATH)aes_crypt_session.cpp \
$(SOURCE_PATH)aes_crypt.cpp \
$(SOURCE_PATH)connection_base.cpp \
$(SOURCE_PATH)controller_segment.cpp \
$(SOURCE_PATH)module_application.cpp \
$(SOURCE_PATH)module_base.cpp \
$(SOURCE_PATH)module_hardware.cpp \
$(SOURCE_PATH)module_system.cpp \
$(SOURCE_PATH)module_table.cpp \
$(SOURCE_PATH)protocol_base.cpp \
$(SOURCE_PATH)protocol_crypt.cpp \
$(SOURCE_PATH)protocol_docking_station_v1.cpp \
$(SOURCE_PATH)protocol.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Datafox File Dateien
SOURCE_PATH = $(SHARED_SRC_PATH)file/
SOURCE_FILES += \
$(SOURCE_PATH)file_device.cpp \
$(SOURCE_PATH)file_device_text.cpp \
$(SOURCE_PATH)file_device_zip.cpp \
$(SOURCE_PATH)file_display_colors.cpp \
$(SOURCE_PATH)file_display_fonts.cpp \
$(SOURCE_PATH)file_display_grafics.cpp \
$(SOURCE_PATH)file_hardware_info.cpp \
$(SOURCE_PATH)file_ini.cpp \
$(SOURCE_PATH)file_intel_hex.cpp \
$(SOURCE_PATH)file_studio_setup.cpp \
$(SOURCE_PATH)file_studio_setup_xml.cpp \
$(SOURCE_PATH)file_touch_keys.cpp \
$(SOURCE_PATH)file_wlan_config.cpp \
$(SOURCE_PATH)ini_file.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer File interface
SOURCE_PATH = $(SHARED_SRC_PATH)file/interface/
SOURCE_FILES += \
$(SOURCE_PATH)data_flash.cpp \
$(SOURCE_PATH)device_update_classes.cpp \
$(SOURCE_PATH)exchange_classes.cpp \
$(SOURCE_PATH)hardware_info_items.cpp \
$(SOURCE_PATH)mcu_memory.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Datafox Items
SOURCE_PATH = $(SHARED_SRC_PATH)file/interface/dfitem/
SOURCE_FILES += \
$(SOURCE_PATH)dfitem_access_control.cpp \
$(SOURCE_PATH)dfitem_access_control_2.cpp \
$(SOURCE_PATH)dfitem_base.cpp \
$(SOURCE_PATH)dfitem_device.cpp \
$(SOURCE_PATH)dfitem_handling.cpp \
$(SOURCE_PATH)dfitem_input_chain.cpp \
$(SOURCE_PATH)dfitem_input_chain_field.cpp \
$(SOURCE_PATH)dfitem_io_module.cpp \
$(SOURCE_PATH)dfitem_mde.cpp \
$(SOURCE_PATH)dfitem_menu.cpp \
$(SOURCE_PATH)dfitem_menu_choice.cpp \
$(SOURCE_PATH)dfitem_table.cpp \
$(SOURCE_PATH)dfitem_table_field.cpp \
$(SOURCE_PATH)dfitem_table_list.cpp \
$(SOURCE_PATH)dfitem_table_record.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Log
SOURCE_PATH = $(SHARED_SRC_PATH)log/
SOURCE_FILES += \
$(SOURCE_PATH)df_log.cpp \
$(SOURCE_PATH)df_log_check.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	g++ $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -std=c++11 -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Static library
$(LIBRARY_FILE_PATH)DFCom.a: $(SOURCE_FILES_O)
	ar -cr $(LIBRARY_FILE_PATH)DFCom.a $(SOURCE_FILES_O)
	ranlib $(LIBRARY_FILE_PATH)DFCom.a

# shared library
#
# Benoetigt zum ausfuehren ggf. Umgebungsvariable: LD_LIBRARY_PATH
# Pruefen ob vorhanden: echo $LD_LIBRARY_PATH
# setzen wenn fehlt:
#    LD_LIBRARY_PATH=/full/path/to/library/directory
#    export LD_LIBRARY_PATH
# oder ergaenzen wenn vorhanden:
#    LD_LIBRARY_PATH=/full/path/to/library/directory:${LD_LIBRARY_PATH}
#    export LD_LIBRARY_PATH
$(LIBRARY_FILE_PATH)libDFCom.so: $(SOURCE_FILES_O)
	g++ -fvisibility=hidden -fPIC -shared $(SOURCE_FILES_O) -o $(LIBRARY_FILE_PATH)libDFCom.so

