// datafox_library_iv.cpp : Definiert die exportierten Funktionen für die DLL-Anwendung.
//

#include "datafox_library_iv.h"
#include "api/df_api.h"
#include "channel/channel_bridge.h"

#include <map>

DatafoxLibraryIV *DatafoxLibraryIV::m_this = NULL;

DatafoxLibraryIV::DatafoxLibraryIV()
{
    m_this = this;

#if defined(_MSC_VER) || defined(__MINGW32__)
    _hModule = NULL;
#endif
    // Members Initialisieren
    setListenerThreadStopSignal( false );
    setListenerThreadState( Stopped );
    setListenerThreadError( IV_ERROR_SUCCESS );
    setListenSocketState( DOWN );

    // Sockets initialisieren
    _listenSocket = INVALID_SOCKET;

    // Verarbeitung von Datensatzmeldungen auf "nicht gesetzt" setzen.
    m_notifyRecord = -1;

    // Globales Passwort fuer die verschluesselte Kommunikation
    memset( m_globalCommunicationPassword, 0, sizeof( m_globalCommunicationPassword ) );
    m_globalCommunicationPasswordSize = 0;
    m_globalSessionTimeout = 0;
}

DatafoxLibraryIV::~DatafoxLibraryIV()
{
    // Aktive Verbindung ggf. beenden.
    stopActiveMode();
    {
        // Map der ChannelBridge leeren.
        //std::lock_guard<std::recursive_mutex> guard( theApp.mutexMapChannelBridge_ );
        for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
            delete iter->second;

        _mapChannelBridge.clear();
    }
}

// Das einzige DatafoxLibraryIV-Objekt

DatafoxLibraryIV theApp;

#if defined(_MSC_VER) || defined(__MINGW32__)
// Lesen der Versionsinformationen aus den Ressourcen
int DatafoxLibraryIV::GetVersion( SoftwarebuildVersion &version )
{
    int res;
    TCHAR szModuleName[MAX_PATH];
    DWORD dwHandle;
    DWORD dwLen;
    LPVOID lpBuffer = NULL;
    unsigned int uiLen;

    unsigned char *pucData = NULL;

    // Modulname der Anwendung
    if ( !GetModuleFileName( theApp._hModule, szModuleName, MAX_PATH ) )
        return 0;

    // Laenge der Version - Resource
    dwLen = GetFileVersionInfoSize( szModuleName, &dwHandle );
    if ( !dwLen )
        return 0;

    pucData = new unsigned char[dwLen + 100]; // unicode
    // Anwendung - Versioninfo ermitteln
    if ( !GetFileVersionInfo( szModuleName, dwHandle, dwLen, reinterpret_cast<LPVOID>( pucData ) ) )
    {
        delete pucData;
        return 0;
    }

    // Pointer auf allgemeine Infostruktur ermitteln
    if ( !VerQueryValue( reinterpret_cast<LPVOID>( pucData ), TEXT( "\\\0" ), &lpBuffer, &uiLen ) )
    {
        delete pucData;
        return 0;
    }
    // Daten zurueckgeben
    VS_FIXEDFILEINFO *pvs_ffi = (VS_FIXEDFILEINFO*)lpBuffer;

    // Ermittelte Version aus der Resource in Versionsobjekt wandeln.
    version.reset();
    res = version.parse( static_cast<unsigned int>( ((pvs_ffi->dwFileVersionMS & 0x00FF0000) << 8) | ((pvs_ffi->dwFileVersionMS & 0x000000FF) << 16) | ((pvs_ffi->dwFileVersionLS & 0x00FF0000) >> 8) | (pvs_ffi->dwFileVersionLS & 0x000000FF) ) );

    delete pucData;

    return res;
}
#endif

/**
* @brief Erhalt eines Kanalobjekts oder NULL bei Bereichsueberschreitung sowie Fehler.
*/
ChannelBridge *DatafoxLibraryIV::channelBridge( int &channelId )
{
    if ( channelBridgeRequest( channelId ) )
    {
        // CallBack fuer Notify uebergeben
        _mapChannelBridge[channelId]->setCallBackNotifyRecord( CBNotifyRecord );
        // Passwort fuer die verschluesselte Kommunikation uebergeben.
        if ( _mapChannelBridge[channelId]->isGlobalCommunicationPassword() )
            _mapChannelBridge[channelId]->setCommunicationPassword( m_globalCommunicationPassword, m_globalCommunicationPasswordSize, m_globalSessionTimeout, 1 );

        return _mapChannelBridge[channelId];
    }

    return nullptr;
}

//#ifdef WIN32
/**
* @brief Sicherstellen das die Anfrage und das Kanalobjekt in Ordnung ist.
*/
bool DatafoxLibraryIV::channelBridgeRequest( int &channelId )
{
    // Kanalbereich pruefen und evntl. erstellen des nicht vorhandenen Kanalobjektes.
    if ( (channelId != -1) && (channelId < 1 || channelId > MAX_CHANNEL) )
    {
        // Bereich falsch
        return false;
    }

    // Bereichssperrung 
    std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );

    // Anfrage auf dynamische Zuweisung?
    if ( channelId == -1 )
    {
        for ( channelId = ACTIVE_CHANNEL_LOWER_BOUND; channelId < ACTIVE_CHANNEL_UPPER_BOUND; ++channelId )
        {
            if  ( _mapChannelBridge.count( channelId ) == 0 )
            {
                break;
            }
            else if ( _mapChannelBridge[channelId]->activeModeState() == Datafox::Communication::ams_unused )
            {
               return true;
            }
        }

        if ( channelId == ACTIVE_CHANNEL_UPPER_BOUND )
        {
            // Alle Kanalklassen sind vergeben!
            channelId = -1;
            return false;
        }
    }
    else
    {
        // Kanalobjekt bereits vorhanden?
        if ( _mapChannelBridge.count( channelId ) )
            return true;
    }


    ChannelBridge *c = new ChannelBridge( static_cast<short>( channelId ) );
    if ( c == nullptr )
        return false;

    // Kanalobjekt erzeugen
    _mapChannelBridge[channelId] = c;

    return true;
}

/**
* @brief Ermitteln ob das Geraet bereits im Active-Mode vorhanden ist.
*/
int DatafoxLibraryIV::searchRequest( ChannelBridge *channel )
{
    int channelId = 0;
    std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );
    if ( channel != nullptr && _mapChannelBridge.size() )
    {
        Channel4 &searchChannel = channel->channel4_;
        for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
        {
            auto currentChannel = iter->second;
            if ( currentChannel->isActive() )
            {
                // Passen der Geraetetyp + serial.
                if (searchChannel.m_hardware_info.typeDevice == currentChannel->channel4_.m_hardware_info.typeDevice 
                    && searchChannel.get_property( "device.serial", -1 ) == currentChannel->channel4_.get_property( "device.serial", -1 ))
                {
                    // Eintrag passt zu gesuchtem.
                    channelId = iter->first;
                    break;
                }
            }            
        }
    }

    return channelId;
}

/**
* @brief Ermitteln an welche Kanalnummer dieser Kanal gebunden werden soll.
*/
int DatafoxLibraryIV::bindRequest( ChannelBridge *channel )
{
    int channelId = 0;
    std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );
    if ( channel != NULL && _mapChannelBridge.size() )
    {
        Channel4 &searchChannel = channel->channel4_;
        for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
        {
            if ( iter->first < ACTIVE_CHANNEL_LOWER_BOUND )
            {
                auto currentChannel = iter->second;
                if ( currentChannel->isBind() )
                {
                    // Soll der Geraetetyp + serial passen oder passt die IP.
                    if ( (searchChannel.m_hardware_info.typeDevice == currentChannel->bindInfo_.typeDevice 
                        && searchChannel.get_property( "device.serial", -1 ) == currentChannel->bindInfo_.serial)
                        || currentChannel->bindInfo_.ip.size() && searchChannel.m_tcpip_info.ip.size()
                        && currentChannel->bindInfo_.ip == searchChannel.m_tcpip_info.ip )
                    {
                        // Statischer Eintrag passt zu gesuchtem.
                        channelId = iter->first;
                        break;
                    }
                }            
            }
            else
            {
                // Eintrag fuer dynamische Zuweisung.
                break;
            }
        }
    }

    return channelId;
}


/**
* @brief Ermitteln an welche Kanalnummer dieser Kanal gebunden werden soll.
*/
bool DatafoxLibraryIV::moveChannelBridge( int channelIDFrom, int channelIDTo )
{
    // Kanalbereiche pruefen.
    if ( channelIDFrom < 1 || channelIDFrom > MAX_CHANNEL
        || channelIDTo < 1 || channelIDTo > MAX_CHANNEL
        || channelIDFrom == channelIDTo )
    {
        // Bereich falsch
        return false;
    }

    std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );

    // Objekt sicherstellen
    if ( _mapChannelBridge.count( channelIDFrom ) == 0 )
        return false;

    // Quellkanal aus Array entkoppeln
    ChannelBridge *channelFrom = _mapChannelBridge[channelIDFrom];
    _mapChannelBridge[channelIDFrom] = nullptr;
    //mapChannelBridge_.erase( channelIDFrom );

    // Zielkanal ggf. Verbindung schliessen und loeschen.
    if ( _mapChannelBridge.count( channelIDTo ) )
    {
        _mapChannelBridge[channelIDTo]->close();
        delete _mapChannelBridge[channelIDTo];
        _mapChannelBridge[channelIDTo] = nullptr;
        //mapChannelBridge_.erase( channelIDTo );
    }

    // Kanal wechseln
    return setChannelBridge( channelIDTo, channelFrom );
}

/**
* @brief Zuweisen eines Kanalobjektes an das Array.
*/
bool DatafoxLibraryIV::setChannelBridge( int channelId, ChannelBridge *channel )
{
    // Kanalbereich pruefen.
    if ( channelId < 1 || channelId > MAX_CHANNEL )
    {
        // Bereich falsch
        return false;
    }

    std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );

    // Kanalobjekt bereits vorhanden?
    if ( _mapChannelBridge.count( channelId ) && _mapChannelBridge[channelId] != nullptr )
    {
        // Loeschen.
        delete _mapChannelBridge[channelId];
        _mapChannelBridge[channelId] = nullptr;
    }

    // Kanalobjekt zuweisen / erzeugen
    _mapChannelBridge[channelId] = channel;

    // Daten zuweisen
    channel->setConnectionId( channelId );

    return true;
}

int DatafoxLibraryIV::startActiveMode( const char *ip, int port, int deviceAddress, int timeout, int aliveTO, int maxConnections, int infoFlags, int reserved, int *piError )
{
    (void)reserved;

    // Nicht bereits aktiv?
    if ( isActiveMode() )
    {
        *piError = IV_ERROR_AC_STARTED;
        return 0;
    }

    strcpy( listenerThreadData.ip, ip );
    listenerThreadData.port = port;
    listenerThreadData.deviceID = deviceAddress;
    listenerThreadData.timeout = timeout;
    listenerThreadData.aliveTO = aliveTO;
    listenerThreadData.maxConnections = maxConnections;
    listenerThreadData.infoFlags = infoFlags;

    // Listener starten.
    setListenerThreadStartSignal();
    _listenerThread = std::thread( [=] { mainListenerThread(); } );
    // Thread umschalten.
    std::this_thread::sleep_for( std::chrono::milliseconds( 10 ) );

    int loops = 0;
    // Start des Threads pruefen.
    while ( listenerThreadStartSignal() == false )
    {
        if ( loops == 100 )
        {
            if ( _listenerThread.joinable() )
            {
                // Nach 1 Sekunde muss der Thread seinen Start signalisiert haben.
                setListenerThreadStopSignal();
                _listenerThread.join();
            }
            break;
        }

        std::this_thread::sleep_for( std::chrono::milliseconds( 10 ) );
        loops++;
    }

    // Wurde ein Fehler ausgeloest.
    if ( listenerThreadError() != IV_ERROR_SUCCESS )
    {
        // Fehler von Workerthread 1:1 weitergeben.
        *piError = listenerThreadError();

        if ( _listenerThread.joinable() )
        {
            // Beendigung sicherstellen.
            setListenerThreadStopSignal();
            _listenerThread.join();
        }
    }

    return isActiveMode();
}

int DatafoxLibraryIV::stopActiveMode()
{
    // Hauptthread deaktivieren, wenn aktiv
    if ( isActiveMode() )
    {
        if ( _listenerThread.joinable() )
        {
            // WorkerThread fuer Socket beenden.
            setListenerThreadStopSignal();
            // Thread Beendigung abwarten.
            _listenerThread.join();
        }

        // Warteschlange loeschen
        if ( IsNotifyRecord() )
        {
            // Bereichssperrung
            std::lock_guard<std::mutex> guard( m_mutexQue );

            while ( m_queue.size() )
                m_queue.pop(); // Eintrag verwerfen
        }
    }

    return (isActiveMode() == false);
}

int DatafoxLibraryIV::SetRecordAvailable( int enable )
{
    // Wert uebernehmen
    m_notifyRecord = (enable == 0) ? 0 : 1;
    // Warteschlange loeschen?
    if ( m_notifyRecord == 0 )
    {
        // Bereichssperrung
        std::lock_guard<std::mutex> guard( m_mutexQue );

        while ( m_queue.size() )
            m_queue.pop(); // Eintrag verwerfen
    }

    return TRUE;
}

bool DatafoxLibraryIV::listenerThreadStartSignal()
{
    return _listenerThreadStartSignal;
}

void DatafoxLibraryIV::setListenerThreadStartSignal( bool signal )
{
    _listenerThreadStartSignal = signal;
}

bool DatafoxLibraryIV::listenerThreadStopSignal()
{
    return _listenerThreadStopSignal;
}

void DatafoxLibraryIV::setListenerThreadStopSignal( bool signal )
{
    _listenerThreadStopSignal = signal;
}

DatafoxLibraryIV::EnumThreadState DatafoxLibraryIV::listenerThreadState()
{
    return _listenerThreadState;
}

void DatafoxLibraryIV::setListenerThreadState( EnumThreadState state )
{
    _listenerThreadState = state;
}

int DatafoxLibraryIV::listenerThreadError()
{
    return _listenerThreadError;
}

void DatafoxLibraryIV::setListenerThreadError( int error )
{
    _listenerThreadError = error;
}

DatafoxLibraryIV::EnumListenSocketState DatafoxLibraryIV::listenSocketState()
{
    return _listenSocketState;
}

void DatafoxLibraryIV::setListenSocketState( EnumListenSocketState state )
{
    _listenSocketState = state;
}

void DatafoxLibraryIV::mainListenerThread()
{
#ifndef CONST
#if defined(_MSC_VER)
#define CONST const
#else
#define CONST
#endif
#endif
    // Zustand auf Starten.
    setListenerThreadStopSignal( false );
    setListenerThreadState( Starting );
    // Kein Fehler setzen.
    setListenerThreadError( IV_ERROR_SUCCESS );
    // Zustandsstatus fuer Listen - Socket
    setListenSocketState( DOWN );

    // Erfolgreichen Start signalisieren.
    setListenerThreadStartSignal( true );

    // Hauptschleife ausfuehren?
    bool run = false;

    int rc;
    int channelID;

    SOCKADDR_IN sa;
    struct timeval seltime;
    fd_set readfds;

    std::map<int, ChannelBridge *> mapAcceptPending;
    bool bindPendingNotify;

    ChannelBridge *channelConnect = NULL;
    Channel4 *channel4Connect = NULL;

    // Locales Channel-Objekt fuer Trace usw..
    channelID = MAX_CHANNEL-1;
    Channel4 &channel = channelBridge( channelID )->channel4_;

    // Maximaler COM.ALIVE von 5 Minuten sicherstellen.
    if ( listenerThreadData.aliveTO > 300 )
    {
        channel.trace( "%s(%d):WARNING(1): %s() [ Zu großer Alive-Timeout (%u), wurde auf 300s gesetzt. ]\n", __FILE__, __LINE__, __func__, listenerThreadData.aliveTO );
        listenerThreadData.aliveTO = 300;
    }

    // Timeout fuer Verbindungsanfragen. 
    //Achtung: Nicht auf 0 da sonst der Thread blockiert wuerde!
    seltime.tv_sec  = 1;
    seltime.tv_usec = 0;

    // Sockethandle reservieren
    _listenSocket = socket( AF_INET, SOCK_STREAM, 0 );
    if ( _listenSocket != INVALID_SOCKET ) 
    {
        memset( &sa, 0, sizeof( sa ) );

        sa.sin_family = AF_INET;
        sa.sin_port = htons( listenerThreadData.port );
#if defined(_MSC_VER) || defined(__MINGW32__)
        sa.sin_addr.S_un.S_addr = inet_addr( listenerThreadData.ip );
#else
        sa.sin_addr.s_addr = inet_addr( listenerThreadData.ip );
#endif

        // Socket mit lokalem Port verbinden.
        if ( ::bind( _listenSocket, reinterpret_cast<SOCKADDR *>( &sa ), sizeof( sa ) ) == SOCKET_ERROR )
        {
#if defined(_MSC_VER) || defined(__MINGW32__)
            if ( WSAGetLastError() == WSAEADDRINUSE )
            {
#else
            if ( WSAGetLastError() == EADDRINUSE )
            {
#endif
                channel.trace( "%s(%i):ERROR(1): %s [ bind fail (%i) Die gewaehlte Portnummer ist in Gebrauch (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError(), listenerThreadData.port );
                // Initialisierungsfehler setzten
                setListenerThreadError( IV_ERROR_AC_PORTINUSE );
            }
            else
            {
                channel.trace( "%s(%i):ERROR(1): %s [ bind fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                setListenerThreadError( IV_ERROR_AC_LISTENER );
            }

#if defined(_MSC_VER) || defined(__MINGW32__)
            if ( closesocket( _listenSocket ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                setListenerThreadError( IV_ERROR_AC_LISTENER );
            }
#else
            if ( shutdown(_listenSocket, 2 ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                setListenerThreadError( IV_ERROR_AC_LISTENER );
            }
            if ( close( _listenSocket ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                setListenerThreadError( IV_ERROR_AC_LISTENER );
            }

#endif

            _listenSocket = INVALID_SOCKET;

            // Zustand auf Fehler.
            setListenerThreadState( Stopping );
        }
        else
        {
            // Mit Reset - Status beginnen
            run = true;
            setListenSocketState( RESET );
            // Zustand auf Laeuft.
            setListenerThreadState( Running );
        }
    }
    else
    {
        channel.trace( "%s(%i):ERROR(1): %s [ socket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
        // Initialisierungsfehler setzten
        setListenerThreadError( IV_ERROR_AC_LISTENER );
        // Zustand auf Fehler.
        setListenerThreadState( Stopping );
    }

    // Hauptschleife des Worker
    while ( run )
    {
        // Beendigungssignal abfragen
        if ( listenerThreadStopSignal() )
        {
            // Zustand setzen
            setListenerThreadState( Stopping );

            // Listensocket herunterfahren
            if ( _listenSocket != INVALID_SOCKET )
            {
                if ( listenSocketState() != DOWN )
                {
                    // Anstehende Verbindungsgesuche verwerfen
                    if ( listen( _listenSocket, 1 ) == SOCKET_ERROR )
                    { 
                        channel.trace("%s(%i):ERROR(1): %s [ listen fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
                }
#if defined(_MSC_VER) || defined(__MINGW32__)
                // Listensocket schliessen
                if ( closesocket( _listenSocket ) == SOCKET_ERROR )
                {
                    channel.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                }
#else
                if ( shutdown( _listenSocket, 2 ) == SOCKET_ERROR )
                {
                    channel.trace( "%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                }
                if ( close( _listenSocket ) == SOCKET_ERROR )
                {
                    channel.trace( "%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                }
#endif
                _listenSocket = INVALID_SOCKET;

                std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );
                // Eventuelle Verbindungen beenden
                for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
                {
                    if ( iter->second->isActive() )
                        iter->second->close();
                }
            }

            // Zustand setzen
            setListenSocketState( DOWN );

            // Hauptschleife verlassen
            run = false;
            continue;
        }

        // Je nach Zustand den Listen-Socket verwalten
        switch ( listenSocketState() )
        {
        case DOWN: 
            // Aufgrund eines Fehlers abgeschaltet
            break;

        case RESET: // Verwirft eventuell anstehende Verbindungsanforderungen,
            // durch (oder) Erzeugung Listensocket.
            if ( listen( _listenSocket, 1 ) == SOCKET_ERROR )
            { 
                channel.trace( "%s(%i):ERROR(1): %s [ listen fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );

                setListenSocketState( DOWN );
                channel.trace( "%s(%i):INFO(1): %s [ State change to down.. ]\n", __FILE__, __LINE__, __func__ );
            } else {

                setListenSocketState( LISTENING );
                channel.trace( "%s(%i):INFO(1): %s [ State change to listening... ]\n", __FILE__, __LINE__, __func__ );
            }
            break;

        case LISTENING: // Wartet auf connect. Im "listening" - Zustand.

            FD_ZERO( reinterpret_cast<fd_set FAR *>( &readfds ) );
            FD_SET( _listenSocket, reinterpret_cast<fd_set FAR *>( &readfds ) );

            if ( (rc = select( FD_SETSIZE,
                reinterpret_cast<fd_set FAR *>( &readfds ), reinterpret_cast<fd_set FAR *>( 0 ), reinterpret_cast<fd_set FAR *>( 0 ),
                &seltime ) ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ select fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
            }
            else if ( rc )
            {
                // Verbindungsanforderung auf Listensocket im naechsten
                // Druchlauf verarbeiten.
                setListenSocketState( ACCEPTING );
                channel.trace( "%s(%i):INFO(1): %s [ State change to accepting... ]\n", __FILE__, __LINE__, __func__ );
            } 
            break;

        case ACCEPTING: { // Verbindungsanforderung verarbeiten

            // Anzahl aktiver Verbindungen ermitteln
            int connected = 0;

            {
                std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );
                for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
                {
                    if ( iter->first < ACTIVE_CHANNEL_UPPER_BOUND )
                    {
                        if ( iter->second->activeModeState() > Datafox::Communication::ams_unused )
                            connected++;
                    }
                    else
                    {
                        break;
                    }
                }
            }

            // Maximale Anzahl Verbinungen noch nicht erreicht?
            channelConnect = NULL;
            channel4Connect = NULL;
            if ( connected < listenerThreadData.maxConnections )
            {
                channelID = -1;
                channelConnect = channelBridge( channelID );
                if ( channelConnect )
                {
                    channel4Connect = &channelConnect->channel4_;
                }
                else
                {
                    setListenSocketState( RESET );
                    continue;
                }
            }
            else
            {
                setListenSocketState( RESET );
                continue;
            }

            channel.trace( "%s(%i):INFO(1): %s [ Try connect to (%i) ]\n", __FILE__, __LINE__, __func__, channelID );

            // Ggf. den Namen für das Tracefile anpassen, ansonsten würde es erst durch einen "normalen" Logaufruf angepasst werden.
            channelConnect->change_trace_file_path( channelConnect->get_current_trace_file_path() );

            if ( !channel4Connect->connect( &_listenSocket, channelID, listenerThreadData.deviceID, listenerThreadData.timeout, listenerThreadData.aliveTO, listenerThreadData.infoFlags ) )
            {
                // Wenn alle Kanalobjekte vergeben wurden
                // oder ein Fehler bei der Erstellung auftrat, dann
                // Verbindungsanforderung verwerfen.
                setListenSocketState( RESET );
                channel.trace( "%s(%i):INFO(1): %s [ State change to reset... ]\n", __FILE__, __LINE__, __func__ );
                continue;
            }

            // Fuer die spaetere weiterverarbeitung merken.
            mapAcceptPending.insert( std::make_pair( channelID, channelConnect ) );

            // Verbindungsanfrage verarbeitet
            setListenSocketState( LISTENING );
            channel.trace( "%s(%i):INFO(1): %s [ State change to listening... ]\n", __FILE__, __LINE__, __func__ );
            break;
            }
        }

        // Alle noch nicht verarbeiteteten Bindings setzen.
        {
            std::lock_guard<std::mutex> guard( _mutexListBindingInfo );
            bindPendingNotify = _listBindPending.size() > 0;
            if ( _listBindPending.size() )
            {
                for ( auto iterBinding = _listBindPending.begin(); iterBinding != _listBindPending.end(); ++iterBinding )
                {
                    ChannelBridge *localChannelBridge = channelBridge( iterBinding->channelId );
                    const StructBindInfo &bindInfo = iterBinding->bi;
                    if ( localChannelBridge )
                    {
                        // Folgende Situationen sind zu Beachten.
                        // 1. Eine dynamische Zuweisung soll statisch werden.
                        // 2. Eine statische Zuweisung soll dynamisch werden.

                        // Aktuelles BindingInfo ist?
                        if ( bindInfo.state == 1 )
                        {
                            if ( localChannelBridge->isBind() == false )
                            {
                                // Bindungsinformation hinterlegen.
                                localChannelBridge->setBind( bindInfo );
                            }
                            else
                            {
                                channel.trace( "%s(%i):INFO(1): %s [ Binding fail for %i: is already bound. ]\n", __FILE__, __LINE__, __func__, iterBinding->channelId );
                            }
                        }
                        else if ( bindInfo.state == 0 )
                        {
                            if ( localChannelBridge->isBind() )
                            {
                                // Bindungsinformation hinerlegen.
                                localChannelBridge->setBind( bindInfo );
                                // Bestehende Verbindung schliessen, Neuanmeldung erforderlich.
                                localChannelBridge->close();
                            }
                            else
                            {
                                // Bindungsinformation hinerlegen.
                                localChannelBridge->setBind( bindInfo );
                            }
                        }
                        else
                        {
                            channel.trace( "%s(%i):INFO(1): %s [ Binding fail for %i: unknown bindInfo.state. ]\n", __FILE__, __LINE__, __func__, iterBinding->channelId );
                        }
                    }
                    else
                    {
                        channel.trace( "%s(%i):INFO(1): %s [ Binding fail for %i: got no channel object. ]\n", __FILE__, __LINE__, __func__, iterBinding->channelId );
                    }
                }

                // Alle Bindungsangaben uebernommen.
                _listBindPending.clear();
                {
                    std::vector<int> deleteChannelBridgeId;
                    std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );
                    // Alle Aktiven Kanaele pruefen
                    for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
                    { 
                        if ( iter->first >= ACTIVE_CHANNEL_LOWER_BOUND && iter->first < ACTIVE_CHANNEL_UPPER_BOUND )
                        {
                            if ( iter->second->isActive() )
                            {
                                int bindId;
                                // Pruefen ob der Kanal an eine andere channelID gebunden werden soll.
                                if ( (bindId = bindRequest( iter->second )) )
                                {
                                    channel.trace( "%s(%i):INFO(1): %s [ Global channel switch %i to %i ]\n", __FILE__, __LINE__, __func__, iter->first, bindId );
                                    // StructBindInfo mitnehmen.
                                    ChannelBridge *bindChannel = channelBridge( bindId );
                                    iter->second->setBind( bindChannel->bindInfo_ );
                                    moveChannelBridge( iter->first, bindId );
                                    // Fuer spaeteres loeschen halten.
                                    deleteChannelBridgeId.push_back( iter->first );
                                }
                            }
                        }
                    }
                    // Liste der zu loeschenden Eintraege.
                    for ( auto iter = deleteChannelBridgeId.begin(); iter != deleteChannelBridgeId.end(); ++iter )
                        _mapChannelBridge.erase( *iter );
                }
            }
        }

        // Alle noch nicht komplett bearbeiteten Verbindungen pruefen.
        if ( mapAcceptPending.size() )
        {
            std::vector<int> deleteAcceptPendingId;

            for ( auto iter = mapAcceptPending.begin(); iter != mapAcceptPending.end(); ++iter )
            {
                if ( iter->second->activeModeState() == Datafox::Communication::ams_usable )
                {
                    // Neue Verbindung erfolgreich entgegengenommen, Bindpruefung durchfuehren.
                    iter->second->setActiveModeState( Datafox::Communication::ams_binding );
                    channel.trace( "%s(%i):INFO(1): %s [ setActiveState(%i) is_binding ]\n", __FILE__, __LINE__, __func__, iter->first );

                    // Bei zwischenzeitlichem Verbindungsverlust eine ggf. vorhandene Kanalnummer wieder zuweisen.
                    int connectionId = searchRequest(iter->second);
                    if (connectionId)
                    {
                        channel.trace("%s(%i):INFO(1): %s [ Local (existing) channel switch %i to %i ]\n", __FILE__, __LINE__, __func__, iter->first, connectionId);
                        // StructBindInfo mitnehmen.
                        ChannelBridge *bindChannel = channelBridge(connectionId);
                        iter->second->setBind(bindChannel->bindInfo_);
                        moveChannelBridge(iter->first, connectionId);
                        // Umgemappten Eintrag loeschen.
                        std::lock_guard<std::recursive_mutex> guard(_mutexMapChannelBridge);
                        _mapChannelBridge.erase(iter->first);
                    }
                    else
                    {
                        int bindId = bindRequest( iter->second );
                        // Pruefen ob der Kanal an eine andere channelID gebunden werden soll.
                        if ( bindId )
                        {
                            channel.trace( "%s(%i):INFO(1): %s [ Local channel switch %i to %i ]\n", __FILE__, __LINE__, __func__, iter->first, bindId );
                            // StructBindInfo mitnehmen.
                            ChannelBridge *bindChannel = channelBridge( bindId );
                            iter->second->setBind( bindChannel->bindInfo_ );
                            moveChannelBridge( iter->first, bindId );
                            // Umgemappten Eintrag loeschen.
                            std::lock_guard<std::recursive_mutex> guard( _mutexMapChannelBridge );
                            _mapChannelBridge.erase( iter->first );
                        }
                    }

                    // Pruefung durchlaufen, ggf. umgemappt.
                    iter->second->setActiveModeState( Datafox::Communication::ams_active );
                    channel.trace( "%s(%i):INFO(1): %s [ setActiveState(%i) is_active ]\n", __FILE__, __LINE__, __func__, iter->first );
                }

                if ( iter->second->activeModeState() == Datafox::Communication::ams_active || iter->second->activeModeState() == Datafox::Communication::ams_unused )
                {
                    if ( iter->second->activeModeState() == Datafox::Communication::ams_unused )
                        channel.trace( "%s(%i):ERROR(1): %s [ accept fail for %i ]\n", __FILE__, __LINE__, __func__, iter->first );

                    // Fuer spaeteres loeschen halten.
                    deleteAcceptPendingId.push_back( iter->first );
                }
            }
            // Liste der zu loeschenden Eintraege.
            for ( auto iter = deleteAcceptPendingId.begin(); iter != deleteAcceptPendingId.end(); ++iter )
                mapAcceptPending.erase( *iter );
        }

        // Wurden Bindungsinformationen verarbeitet.
        if (bindPendingNotify)
        {
            std::lock_guard<std::mutex> guard( _mutexListBindingInfo );
            if (_listBindPending.size() > 0)
                _conditionListBindingInfo.notify_one();
            else
                _conditionListBindingInfo.notify_all();
        }

        // Kurze Ausszeit um die Systemlast zu reduzieren.
        std::this_thread::sleep_for( std::chrono::milliseconds( 10 ) );
    }

    // Thread Beendigung ruecksetzen, lediglich Fehlerzustand bleibt erhalten.
    setListenerThreadState( Stopped );
}

/*! Passwort fuer die verschluesselte Kommunikation setzen.
\param[in] channelID Auf welche Kanalnummer die Vergabe anzwenden ist. 
Wird dieser Parameter mit -1 übergeben bezieht sich das Passwort auf alle Kanäle, 
deviceID wird dann ebenfalls wie -1 behandelt.
\param[in] deviceID 

*/
int DatafoxLibraryIV::SetCommunicationPassword( int channelID, const char *password, int size, int sessionTimeout )
{
    if ( channelID == -1 )
    {
        if ( size == 0 )
        {
            // Versuchen als Zeichenfolge auszuwerten
            for ( ; size < 16 && password[size]; ++size );
        }
        else if ( size < 0 || size > 16 )
        {
            return 0;
        }

        if ( size == 0 )
        {
            // Ggf. Globale Passwortangabe loeschen
            memset( m_globalCommunicationPassword, 0, sizeof( m_globalCommunicationPassword ) );
            m_globalCommunicationPasswordSize = 0;
            m_globalSessionTimeout = 0; // 1 Stunde default.
        }
        else
        {
            memset( m_globalCommunicationPassword, 0, sizeof( m_globalCommunicationPassword ) );
            memcpy( m_globalCommunicationPassword, password, size );
            m_globalCommunicationPasswordSize = size;
            m_globalSessionTimeout = sessionTimeout;
        }

        // Alle verfuegbaren Kanaele durchgehen und Passwort vergeben.
        std::lock_guard<std::recursive_mutex> guard( theApp._mutexMapChannelBridge );
        for ( auto iter = _mapChannelBridge.begin(); iter != _mapChannelBridge.end(); ++iter )
        {
            // Passwort an die Kanalklasse weitergeben
            if ( iter->second->isGlobalCommunicationPassword() )
                iter->second->setCommunicationPassword( m_globalCommunicationPassword, m_globalCommunicationPasswordSize, m_globalSessionTimeout, 1 );

        }
    }
    else
    {

        ChannelBridge *pChannel = channelBridge( channelID );
        if ( pChannel == 0 )
            return 0;

        if ( size == 0 )
        {
            // Versuchen als Zeichenfolge auszuwerten
            for ( ; size < 16 && password[size]; ++size );
        }
        else if ( size < 0 || size > 16 )
        {
            return 0;
        }

        // Es wird ein Passwort direckt fuer diesen Kanal gesetzt. Wenn das Passwort durch size == 0 geloescht
        // werden soll, dann muss es wieder moeglich sein dem Kanal ueber Global ein Passwort zuzuweisen.
        pChannel->setCommunicationPassword( password, size, sessionTimeout, (size == 0) );
    }

    return 1;
}
