/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "datafox/communication/module_system.h"

#include <memory>
#include <stdlib.h>
#include <ctype.h>
#include <sstream>
#include <string>
#include <boost/scoped_array.hpp>
#include <boost/algorithm/string.hpp>
#include <regex>
#include <boost/format.hpp>
#include <iomanip>
#include <array>

#include "common/software_version.h"

// Konvertiert latin1 zu GSM 7bit character set
void Datafox::Communication::Module_system::latin1_to_gsm_codec( std::string &buffer )
{
    std::string lBuf;
    std::string::value_type c1 = '\0';
    std::string::value_type c2 = '\0';
    std::string::value_type c3 = '\0';
    std::string::value_type c4 = '\0';

    for ( std::size_t i = 0; i < buffer.size(); ++i )
    {
        switch ( (uint8_t)buffer[i] )
        {
        case 0x40: lBuf += "\\00"; break;
        case 0xA3: lBuf += "\\01"; break;
        case 0x24: lBuf += "\\02"; break;
        case 0xA5: lBuf += "\\03"; break;
        case 0xE8: lBuf += "\\04"; break;
        case 0xE9: lBuf += "\\05"; break;
        case 0xF9: lBuf += "\\06"; break;
        case 0xEC: lBuf += "\\07"; break;
        case 0xF2: lBuf += "\\08"; break;
        case 0xC7: lBuf += "\\09"; break;
        case 0xD8: lBuf += "\\0B"; break;
        case 0xF8: lBuf += "\\0C"; break;
        case 0xC5: lBuf += "\\0E"; break;
        case 0xE5: lBuf += "\\0F"; break;
        case 0x5F: lBuf += "\\11"; break;
        case 0x0C: lBuf += "\\1B0A"; break;
        case 0x5E: lBuf += "\\1B14"; break;
        case 0x7B: lBuf += "\\1B28"; break;
        case 0x7D: lBuf += "\\1B29"; break;
        case 0x5C: {
            size_t len = buffer.size() - i;
            if ( len > 2 )
            {
                c1 = std::toupper( buffer[i + 1], std::locale() );
                c2 = std::toupper( buffer[i + 2], std::locale() );
                if  ( len > 4 )
                {
                    c3 = std::toupper( buffer[i + 3], std::locale() );
                    c4 = std::toupper( buffer[i + 4], std::locale() );
                }
                // Handelt es sich um eine bekannte Maskierung?
                // Dann wird diese mit übernommen,
                // ansonsten wird der \ maskiert.
                if ( c1 == '0' )
                {
                    if ( c2 == '0' || c2 == '1' || c2 == '2' || c2 == '3'
                        || c2 == '4' || c2 == '5' || c2 == '6' || c2 == '7'
                        || c2 == '8' || c2 == '9' || c2 == 'B' || c2 == 'C'
                        || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        continue;
                    }
                }
                else if ( (len > 4) && c1 == '1' && c2 == 'B' )
                {
                    if ( (c3 == '0' && c4 == 'A')
                        || (c3 == '1' && c4 == '4')
                        || (c3 == '2' && c4 == '8')
                        || (c3 == '2' && c4 == '9')
                        || (c3 == '2' && c4 == 'F')
                        || (c3 == '3' && c4 == 'C')
                        || (c3 == '3' && c4 == 'D')
                        || (c3 == '3' && c4 == 'E')
                        || (c3 == '4' && c4 == '0') )
                    {
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        continue;
                    }

                }
                else if ( c1 == '1' )
                {
                    if ( c2 == '1' || c2 == 'C' || c2 == 'D' || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        continue;
                    }
                }
                else if ( (c1 == '2' && c2 == '4')
                    || (c1 == '4' && c2 == '0')
                    || (c1 == '6' && c2 == '0') )
                {
                    lBuf.push_back( buffer[i] );
                    i++;
                    lBuf.push_back( buffer[i] );
                    i++;
                    lBuf.push_back( buffer[i] );
                    continue;
                }
                else if ( c1 == '5' )
                {
                    if ( c2 == 'B' || c2 == 'C' || c2 == 'D' || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        continue;
                    }
                }
                else if ( c1 == '7' )
                {
                    if ( c2 == 'B' || c2 == 'C' || c2 == 'D' || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        i++;
                        lBuf.push_back( buffer[i] );
                        continue;
                    }
                }
            }
            lBuf += "\\1B2F"; 
            break;}
        case 0x5B: lBuf += "\\1B3C"; break;
        case 0x7E: lBuf += "\\1B3D"; break;
        case 0x5D: lBuf += "\\1B3E"; break;
        case 0x7C: lBuf += "\\1B40"; break;
        case 0xC6: lBuf += "\\1C"; break;
        case 0xE6: lBuf += "\\1D"; break;
        case 0xDF: lBuf += "\\1E"; break;
        case 0xC9: lBuf += "\\1F"; break;
        case 0xA4: lBuf += "\\24"; break;
        case 0xA1: lBuf += "\\40"; break;
        case 0xC4: lBuf += "\\5B"; break;
        case 0xD6: lBuf += "\\5C"; break;
        case 0xD1: lBuf += "\\5D"; break;
        case 0xDC: lBuf += "\\5E"; break;
        case 0xA7: lBuf += "\\5F"; break;
        case 0xBF: lBuf += "\\60"; break;
        case 0xE4: lBuf += "\\7B"; break;
        case 0xF6: lBuf += "\\7C"; break;
        case 0xF1: lBuf += "\\7D"; break;
        case 0xFC: lBuf += "\\7E"; break;
        case 0xE0: lBuf += "\\7F"; break;
        default: lBuf.push_back( buffer[i] );
        }
    }

    // Ggf. geänderten Wert übernehmen
    buffer = lBuf;
}

// Konvertiert GSM 7bit character set zu latin1
void Datafox::Communication::Module_system::gsm_to_latin1_codec( std::string &buffer )
{
    std::string lBuf;
    std::string::value_type c1 = '\0';
    std::string::value_type c2 = '\0';
    std::string::value_type c3 = '\0';
    std::string::value_type c4 = '\0';

    for ( std::size_t i = 0; i < buffer.size(); ++i )
    {
        if ( ((buffer.size() - i) > 2) && buffer[i] == '\\' ) 
        {
            c1 = std::toupper( buffer[i + 1], std::locale() );
            c2 = std::toupper( buffer[i + 2], std::locale() );
            if ( (buffer.size() - i) > 4 )
            {
                c3 = std::toupper( buffer[i + 3], std::locale() );
                c4 = std::toupper( buffer[i + 4], std::locale() );
            }

            if( c1 == '0' )
            {
                if ( c2 == '0' )
                {
                    lBuf.push_back( 0x40 );
                    i += 2;
                }
                else if ( c2 == '1' )
                {
                    lBuf.push_back( (char)0xA3 );
                    i += 2;
                }
                else if ( c2 == '2' )
                {
                    lBuf.push_back( 0x24 );
                    i += 2;
                }
                else if ( c2 == '3' )
                {
                    lBuf.push_back( (char)0xA5 );
                    i += 2;
                }
                else if ( c2 == '4' )
                {
                    lBuf.push_back( (char)0xE8 );
                    i += 2;
                }
                else if ( c2 == '5' )
                {
                    lBuf.push_back( (char)0xE9 );
                    i += 2;
                }
                else if ( c2 == '6' )
                {
                    lBuf.push_back( (char)0xF9 );
                    i += 2;
                }
                else if ( c2 == '7' )
                {
                    lBuf.push_back( (char)0xEC );
                    i += 2;
                }
                else if ( c2 == '8' )
                {
                    lBuf.push_back( (char)0xF2 );
                    i += 2;
                }
                else if ( c2 == '9' )
                {
                    lBuf.push_back( (char)0xC7 );
                    i += 2;
                }
                else if ( c2 == 'B' )
                {
                    lBuf.push_back( (char)0xD8 );
                    i += 2;
                }
                else if ( c2 == 'C' )
                {
                    lBuf.push_back( (char)0xF8 );
                    i += 2;
                }
                else if ( c2 == 'E' )
                {
                    lBuf.push_back( (char)0xC5 );
                    i += 2;
                }
                else if ( c2 == 'F' )
                {
                    lBuf.push_back( (char)0xE5 );
                    i += 2;
                }
                else
                {
                    lBuf.push_back( buffer[i] );
                }
            }
            else if ( c1 == '1' )
            {
                if ( c2 == '1' )
                { 
                    lBuf.push_back( 0x5F );
                    i += 2;
                }
                else if ( c2 == 'B' )
                {
                    if ( c3 == '0' && c4 == 'A' )
                    {
                        lBuf.push_back( 0x0C );
                        i += 4;
                    }
                    else if ( c3 == '1' && c4 == '4' )
                    {
                        lBuf.push_back( 0x5E );
                        i += 4;
                    }
                    else if ( c3 == '2' && c4 == '8' )
                    {
                        lBuf.push_back( 0x7B );
                        i += 4;
                    }
                    else if ( c3 == '2' && c4 == '9' )
                    {
                        lBuf.push_back( 0x7D );
                        i += 4;
                    }
                    else if ( c3 == '2' && c4 == 'F' )
                    {
                        lBuf.push_back( 0x5C );
                        i += 4;
                    }
                    else if ( c3 == '3' && c4 == 'C' )
                    {
                        lBuf.push_back( 0x5B );
                        i += 4;
                    }
                    else if ( c3 == '3' && c4 == 'D' )
                    {
                        lBuf.push_back( 0x7E );
                        i += 4;
                    }
                    else if ( c3 == '3' && c4 == 'E' )
                    {
                        lBuf.push_back( 0x5D );
                        i += 4;
                    }
                    else if ( c3 == '4' && c4 == '0' )
                    {
                        lBuf.push_back( 0x7C );
                        i += 4;
                    }
                    else
                    {
                        lBuf.push_back( buffer[i] );
                    }
                }
                else if ( c2 == 'C' )
                {
                    lBuf.push_back( (char)0xC6 );
                    i += 2;
                }
                else if ( c2 == 'D' )
                {
                    lBuf.push_back( (char)0xE6 );
                    i += 2;
                }
                else if ( c2 == 'E' )
                {
                    lBuf.push_back( (char)0xDF );
                    i += 2;
                }
                else if ( c2 == 'F' )
                {
                    lBuf.push_back( (char)0xC9 );
                    i += 2;
                }
                else
                {
                    lBuf.push_back( buffer[i] );
                }
            }
            else if ( c1 == '2' )
            {
                if ( c2 == '4' )
                {
                    lBuf.push_back( (char)0xA4 );
                    i += 2;
                }
                else
                {
                    lBuf.push_back( buffer[i] );
                }
            }
            else if ( c1 == '4' )
            {
                if ( c2 == '0' )
                {
                    lBuf.push_back( (char)0xA1 );
                    i += 2;
                }
                else
                {
                    lBuf.push_back(buffer[i]);
                }
            }
            else if ( c1 == '5' )
            {
                if ( c2 == 'B' )
                {
                    lBuf.push_back( (char)0xC4 );
                    i += 2;
                }
                else if ( c2 == 'C' )
                {
                    lBuf.push_back( (char)0xD6 );
                    i += 2;
                }
                else if ( c2 == 'D' )
                {
                    lBuf.push_back( (char)0xD1 );
                    i += 2;
                }
                else if ( c2 == 'E' )
                {
                    lBuf.push_back( (char)0xDC );
                    i += 2;
                }
                else if ( c2 == 'F' )
                {
                    lBuf.push_back( (char)0xA7 );
                    i += 2;
                }
                else
                {
                    lBuf.push_back( buffer[i] );
                }
            }
            else if ( c1 == '6' )
            {
                if ( c2 == '0' )
                {
                    lBuf.push_back( (char)0xBF );
                    i += 2;
                }
                else
                {
                    lBuf.push_back( buffer[i] );
                }
            }
            else if ( c1 == '7' )
            {
                if ( c2 == 'B' )
                {
                    lBuf.push_back( (char)0xE4 );
                    i += 2;
                }
                else if ( c2 == 'C' )
                {
                    lBuf.push_back( (char)0xF6 );
                    i += 2;
                }
                else if ( c2 == 'D' )
                {
                    lBuf.push_back( (char)0xF1 );
                    i += 2;
                }
                else if ( c2 == 'E' )
                {
                    lBuf.push_back( (char)0xFC );
                    i += 2;
                }
                else if ( c2 == 'F' )
                {
                    lBuf.push_back( (char)0xE0 );
                    i += 2;
                }
                else
                {
                    lBuf.push_back( buffer[i] );
                }
            }
            else
            {
                lBuf.push_back( buffer[i] );
            }
        } 
        else 
        {
            lBuf.push_back( buffer[i] );
        }
    }

    // Ggf. geänderten Wert übernehmen
    buffer = lBuf;
}


// ##################################################################
/**
* @brief Initialisieren der Membervariablen.
*
* @param id Nummer der Kanalklasse fuer z.B. Trace.
*/
Datafox::Communication::Module_system::Module_system( int connection_id, uint8_t device_address )
    : Module_hardware( connection_id, device_address ) 
{
    // Passwort fuer die Kommunikation
    m_password_communication.clear();
    m_is_password_protected = 0;
    m_is_password_protection = 0;

    // Firmwareversion des Geraetes.
    m_version_firmware.clear();
    
    // Transparentmodus
    m_transparency_mode = tm_off;

    // Streamtransfer
    m_stream_buffer.clear();
    m_stream_transfer = false;
}

// ##################################################################
/**
* @brief Destruktor.
*
*/
Datafox::Communication::Module_system::~Module_system() 
{
}

// ##################################################################
/**
 * @brief Lesen aus der Kommunikationsschnittstelle.
 */
int Datafox::Communication::Module_system::read_data( uint8_t *buffer, std::size_t read_count, std::size_t &read_in_count, uint8_t read_trials )
{
    trace( "%s(%i):INFO(3): %s( %p, %u, %u, %u )\n", __FILE__, __LINE__, __func__, buffer, read_count, read_in_count, read_trials );

    int res;
    switch ( m_transparency_mode )
    {
        case tm_off:
        case tm_raw:
            res = get_connection().read_data( buffer, read_count, read_in_count, read_trials );
            break;

        case tm_masked:
        {
            uint8_t dataByte;
            char *data = reinterpret_cast<char *>( buffer );
            std::size_t dataByteCount;
            bool masked = false;
            int maskedCount = 0;
#ifdef _DEBUG
            clock_t clockStart = clock();
#endif
            // Datenstrom zeichenweise bearbeiten und besondere Zeichen maskieren
            for ( read_in_count = 0; read_in_count < read_count; ++read_in_count )
            {
                if ( (res = get_connection().read_data( &dataByte, 1, dataByteCount, read_trials )) != IV_ERROR_SUCCESS )
                {
#ifdef _DEBUG
                    std::string prefix = boost::str( boost::format( "(%4.3f)\n" ) % (static_cast<double>(clock() - clockStart) / CLOCKS_PER_SEC) );
                    trace( prefix.c_str() );
#endif
                    return res;
                }
#ifdef _DEBUG
                if ( read_in_count == (read_count - 1) && dataByte != IV_PACKET_SIGN_ESCAPE )
                    trace_data( (uint8_t *)&dataByte, 1, tdm_end );
                else if ( read_in_count == 0 )
                    trace_data( (uint8_t *)&dataByte, 1, tdm_begin );
                else
                    trace_data( (uint8_t *)&dataByte, 1, tdm_append );
#endif
                if ( dataByte == '~' )
                {
                    m_transparency_mode = tm_off;
                }

                if ( dataByte == IV_PACKET_SIGN_ESCAPE )
                {
                    masked = true;
                    read_in_count--;
                } 
                else if ( masked )
                {
                    if ( dataByte == 0x00 )
                        data[read_in_count] = IV_PACKET_SIGN_ESCAPE;
                    else if ( dataByte == 0x01 )
                        data[read_in_count] = IV_PACKET_SIGN_START;
                    else
                        data[read_in_count] = dataByte;

                    maskedCount++;
                    masked = false;
                }
                else
                {
                    data[read_in_count] = dataByte;
                }
            }
#ifdef _DEBUG
            std::string prefix = boost::str( boost::format( "(%4.3f)\n" ) % (double(clock() - clockStart) / CLOCKS_PER_SEC) );
            trace( prefix.c_str() );
#endif
            break;
        }

        default:
            trace( "%s(%i):ERROR(1): %s() [ Unbekannter Mode (%i) ]\n", __FILE__, __LINE__, __func__, m_transparency_mode );
            return -IV_ERROR_TRANSMIT;
    }

    return res;
}

// ##################################################################
/**
 * @brief Schreiben in die Kommunikationsschnittstelle.
 */
int Datafox::Communication::Module_system::write_data( const uint8_t *buffer, std::size_t write_count, std::size_t &write_out_count, uint8_t write_trials )
{
    trace( "%s(%i):INFO(3): %s( %p, %u, %u, %u )\n", __FILE__, __LINE__, __func__, buffer, write_count, write_out_count, write_trials );

    int res;
    switch ( m_transparency_mode )
    {
        case tm_off:
        case tm_raw:
            res = get_connection().write_data( buffer, write_count, write_out_count, write_trials );
            break;

        case tm_masked:
        {
            // Für das Versenden als gesamten Block, die Daten maskiert in einen stringstream packen.
            std::vector<uint8_t> v;
            const uint8_t *data = (const uint8_t *)buffer;
            std::size_t masked = 0;
#ifdef _DEBUG
            clock_t clockStart = clock();
#endif
            // Datenstrom zeichenweise bearbeiten und besondere Zeichen maskieren
            for ( std::size_t count = 0; count < write_count; ++count )
            {
                // Paketzeichen im Datenstrom mit Escapesequenz umschreiben
                if ( (data[count] == IV_PACKET_SIGN_START) || (data[count] == IV_PACKET_SIGN_ESCAPE) )
                {
                    v.push_back( IV_PACKET_SIGN_ESCAPE );
                    // Anzahl maskierte mitzaehlen
                    masked++;

                    switch ( data[count] ) 
                    {
                    case IV_PACKET_SIGN_START:
                        v.push_back( 0x01 );
                        break;

                    case IV_PACKET_SIGN_ESCAPE:
                        v.push_back( 0x00 );
                        break;
                    }
                } 
                else 
                {
                    v.push_back( data[count] );
                }
            }
#ifdef _DEBUG  
            std::string prefix = boost::str( boost::format( "(%4.3f) write:" ) % (double(clock() - clockStart) / CLOCKS_PER_SEC) );
            trace_data( (uint8_t *)v.data(), v.size(), tdm_full_with_crlf, prefix.c_str() );
#endif
            res = get_connection().write_data( v.data(), v.size(), write_out_count, write_trials );
            write_out_count -= masked; // Die Maskierungszeichen zählen nicht als mitgesendet.
            break;
        }

        default:
            trace( "%s(%i):ERROR(1): %s() [ Unbekannter Mode (%i) ]\n", __FILE__, __LINE__, __func__, m_transparency_mode );
            return -IV_ERROR_TRANSMIT;
    }

    return res;
}

// ##################################################################
/**
* @brief Remoteaufruf einer lokalen Modulfunktion.
*/
void Datafox::Communication::Module_system::trace_data( uint8_t *data, std::size_t size, enum_trace_data_mode mode, const char *prefix )
{
    if ( size > 0 )
    {
        // Byte-Array in Hex-String wandeln und ausgeben.
        std::string buf;

        for ( int len = 0; len < size; ++len )
        {
            if ( len == 0 && (mode == tdm_full || mode == tdm_full_with_crlf || mode == tdm_begin) )
            {
                // Prefix zuerst ausgeben?
                if ( prefix != nullptr )
                    buf = prefix;
                
                buf += boost::str( boost::format( "[%1$02X " ) % (int32_t)data[len] );
            }
            else if ( len == (size - 1) && (mode == tdm_full || mode == tdm_full_with_crlf || mode == tdm_end || mode == tdm_end_with_crlf) )
            {
                buf += boost::str( boost::format( "%1$02X]" ) % (int32_t)data[len] );
                // CrLf anfügen?
                if ( mode == tdm_full_with_crlf || mode == tdm_end_with_crlf )
                    buf += "\n";                
            }
            else
            {
                buf += boost::str( boost::format( "%1$02X " ) % (int32_t)data[len] );
            }
        }

        trace( buf.c_str() );
    }
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Datafox::Communication::Module_system::error_module( uint16_t module, uint16_t error, std::string &text )
{
    trace("%s(%i):INFO(3): %s( %u, %u, %p )\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_SYSTEM ?
    if ( module == IV_MODULE_SYSTEM )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Module_hardware::error_module( module, error, text );
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Datafox::Communication::Module_system::error_function( uint16_t module, uint16_t func, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %u, %u, %u, %p )\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_SYSTEM )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 ) 
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %u ): Korrektes Passwort für die Kommunikation erforderlich.") % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %u ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt.") % error );
            return;
        }

        switch ( func )
        {
        case IV_FUNCTION_SYSTEM_RESET:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Resetmodus unbekannt.") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_SERIAL:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Pruefung der Seriennummer - Verschluesselung schlug fehl.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_GET_VERSION:
        case IV_FUNCTION_GET_VERSION_BOOTLOADER:
        case IV_FUNCTION_GET_SERIAL:
        case IV_FUNCTION_SET_RTC:
        case IV_FUNCTION_GET_RTC:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): ") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_SYSTEM_VARIABLE:
        case IV_FUNCTION_GET_SYSTEM_VARIABLE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Der Majorbestandteil des Variablennamens ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %u ): Der Minorbestandteil des Variablennamens ist unbekannt.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %u ): Der verwendete Variablenwert ist zu lang.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %u ): Der verwendete Variablentype würde nicht eingehalten.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %u ): Das Schreib-/Leserecht würde nicht eingehalten.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %u ): Das verwendete Format oder der Wertebereich würde nicht eingehalten.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_RELAY:
        case IV_FUNCTION_GET_RELAY:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Bereichsname unbekannt. (INTERN, TMR, IOM ...)") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %u ): Busnummer (z.B. TMR) nicht bekannt oder erreichbar.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %u ): Relaisnummer unbekannt.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %u ): Zu setzender Zustand (Status) unbekannt oder nicht unterstützt.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_MMC:
        case IV_FUNCTION_SET_FLASH:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Speicherseite konnte nicht geschrieben werden.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %u ): Speicherplatz reicht nicht aus.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_GET_SFSMPB:
        case IV_FUNCTION_GET_MMC:
        case IV_FUNCTION_GET_FLASH:
        case IV_FUNCTION_SET_EEPROM:
        case IV_FUNCTION_GET_EEPROM:
        case IV_FUNCTION_SET_PROGRAM:
        case IV_FUNCTION_GET_PROGRAM:
        case IV_FUNCTION_GET_RAM:
        case IV_FUNCTION_SET_RAM:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): ") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_SET_SEGMENT:
        case IV_FUNCTION_END_SET_SEGMENT:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Der Typ ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %u ): Die Updatesegmente sind erschöpft.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %u ): Der Angeforderte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %u ): Es wurde kein STOP vorm START durchgeführt.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %u ): Verschachtelungsfehler von STOP und START.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %u ): Prüfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener überein.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_SET_SEGMENT_OK:
        case IV_FUNCTION_END_SET_SEGMENT_OK:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Der Benötigte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_GET_SEGMENT:
        case IV_FUNCTION_END_GET_SEGMENT:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Der Typ ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %u ): Die Updatesegmente sind erschöpft.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %u ): Der Benötigte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %u ): Es wurde kein STOP vorm START durchgeführt.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %u ): Verschachtelungsfehler von STOP und START.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %u ): Prüfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener überein.") % error );
                break;

            case 12:
                text = boost::str( boost::format( "( %u ): File not found." ) % error );
                break;

            case 13:
                text = boost::str( boost::format( "( %u ): File will be created." ) % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_GET_SEGMENT_OK:
        case IV_FUNCTION_END_GET_SEGMENT_OK:
            break;

        case IV_FUNCTION_ENTER_BIOS:
        case IV_FUNCTION_EXIT_BIOS:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): ") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_ENTERED_BIOS:
        case IV_FUNCTION_EXITED_BIOS:
            break;

        case IV_FUNCTION_GET_INFO:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Der Majorbestandteil des Modulnamens ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %u ): Der Minorbestandteil des Modulnamens ist unbekannt.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %u ): Es wurden zu wenige Paramteter angegeben.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %u ): Es wurden zu viele Paramteter angegeben.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %u ): Der Wert eines Parameters liegt auserhalb seines Gültigkeitsbereichs.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %u ): Die Information zur Komponente steht nicht zur Verfügung.") % error );
                break;

            case 7:
                text = boost::str( boost::format( "( %u ): Das Modul kann zur Zeit nicht angesprochen werden.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_OPEN_COM_SERVER_MODE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Es wurden zu wenige Paramteter angegeben.") % error );
                break;
            case 2:
                text = boost::str( boost::format( "( %u ): Es wurden zu viele Paramteter angegeben.") % error );
                break;
            case 3:
                text = boost::str( boost::format( "( %u ): Die Angabe des comm Parameters wird nicht unterstützt.") % error );
                break;
            case 4:
                text = boost::str( boost::format( "( %u ): Die Angabe des baudRate Parameters wird nicht unterstützt.") % error );
                break;
            case 5:
                text = boost::str( boost::format( "( %u ): Die Angabe des byteSize Parameters wird nicht unterstützt.") % error );
                break;
            case 6:
                text = boost::str( boost::format( "( %u ): Die Angabe des parity Paramters wird nicht unterstützt.") % error );
                break;
            case 7:
                text = boost::str( boost::format( "( %u ): Die Angabe des stopBits Paramters wird nicht unterstützt.") % error );
                break;
            case 8:
                text = boost::str( boost::format( "( %u ): Die Angabe des flags Parameters wird nicht unterstützt.") % error );
                break;
            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_CLOSE_COM_SERVER_MODE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Bei der Übertragung ist ein Overflow eingetreten.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_OPEN_COM_SERVER_MODE_OK:
            break;

        case IV_FUNCTION_PASSWORD_CHECK:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Passwort stimmt nicht mit dem benötigten überein.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_TRANSMIT_STREAM:
            switch ( error )
            {
            case 3:
                text = boost::str( boost::format( "( %u ): Der Angeforderte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_TRANSPARENCY_MODE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %u ): Der Angeforderte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            case 2: // Kann kommen wenn das Flag-Bit3 sitzt. Es wird damit ein TimeboyI-III gefordert.
                text = boost::str( boost::format( "( %u ): Es wird versucht ein TimeboyIV anzusprechen.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        default: 
            text = boost::str( boost::format( "( %u ): Nicht eingetragener Fehlerblock für Funktion!") % error );
            break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Module_hardware::error_function( module, func, error, text );
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
// Modulbefehl senden und Antwort zurueckgeben
//   -> Adresse (Geraetenummer) des Empfaengers
//  module   -> Nummer des anzusprechenden Modules
//  func -> Nummer der Funktion im Module
//	buf	     -> Buffer fuer Funktionsparameter.
//	count    -> Anzahl der Funktionsparameter in Byte. Rueckgabewert bei Parameterempfang.
//
//  return -> <0 -> Kommunikationsfehler
//             0 -> senden und empfangen und ausfuehren erfolgreich
//			  >0 -> module-/Funktionsfehler
int Datafox::Communication::Module_system::call( uint16_t context, uint16_t module, uint16_t func, uint8_t *buffer, uint16_t &count, int context_value )
{
    trace( "%s(%i):INFO(3): %s( %u, %u, %u, %p, %u, %i )\n", __FILE__, __LINE__, __func__, context, module, func, buffer, count, context_value );

    // Aufruf schaltet den Transparent-Modus ab.
    if ( is_transparency_mode() )
        m_transparency_mode = tm_off;

    int result = Module_base::call( context, module, func, buffer, count, context_value );

    // Bei Verwendung eines Passworts:
    // Wenn das Passwort noch nicht übertragen wurde, wird der Fehler hier abgefangen.
    // Das Passwort wird übertragen und der geforderte Befehl erneut ausgeführt.
    if ( (result & 0xFF) == 28 && get_password_communication().size() )
    {
        trace( "%s(%i):INFO(1): %s() [ Passwort wird gefordert. ]\n", __FILE__, __LINE__, __func__, context, module, func, buffer, count, context_value );

        int lRes;
        // Buffer für Folgebefehl zwischenspeichern,
        // da das hinter buf liegende Bytearray durch Aufruf von passwordCheck verändert wird.
        uint8_t data[IV_PACKET_LENGTH_DATA];
        memset( data, 0, sizeof(data) );
        memcpy( data, buffer, count );

        // Passwortprüfung durchführen
        if ( (lRes = write_setup_password( get_device_address_receiver(), get_password_communication() )) == IV_ERROR_SUCCESS )
        {
            // Passwortprüfung erfolgreich, Parameterdaten wiederherstellen.
            memcpy( buffer, data, count );
            // Befehl erneut ausführen.
            return Module_base::call( context, module, func, buffer, count, context_value );
        }
        else
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK, result );
            // Passwortprüfung fehlgeschlagen, Parameterdaten wiederherstellen.
            memcpy( buffer, data, count );
        }

        // Vorherigen Fehler zurückliefern.
        return result;
    }

    return result;
}

//#########################################################
//################# HARDWARE BUGFIX #######################
//#########################################################

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und stellt einen Bugfix da.
*
*/
int Datafox::Communication::Module_system::read_display_type( uint8_t device_address, uint8_t &val, bool checked )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    int res;
    std::string version;
    // Befindet sich im Bootloader?
    if ( !checked && (res = read_firmware_version( device_address, version )) != IV_ERROR_SUCCESS )
    {
        if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
        {
            // Befehl wird nicht unterstuetzt
            // Geraet befindet sich im Bootloadermodus
            if ( (res = read_bootloader_version( device_address, version )) != IV_ERROR_SUCCESS )
            {
                return res;
            }

            // Ist eine von den gesuchten Versionen?
            if ( version.find("00.04.00.00") != std::string::npos || version.find("00.04.00.01") != std::string::npos )
            {
                // Auf entsprechende Funktion verzweigen
                return Module_hardware::read_transponder_reader_type( device_address, val, true );
            }
        }
    }

    return Module_hardware::read_display_type( device_address, val, true );
}

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und stellt einen Bugfix da.
*
*/
int Datafox::Communication::Module_system::read_keypad_type( uint8_t device_address, uint8_t &val, bool checked )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    int res;
    std::string version;
    // Befindet sich im Bootloader?
    if ( !checked && (res = read_firmware_version( device_address, version )) != IV_ERROR_SUCCESS )
    {
        if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
        {
            // Befehl wird nicht unterstuetzt
            // Geraet befindet sich im Bootloadermodus
            if ( (res = read_bootloader_version( device_address, version )) != IV_ERROR_SUCCESS )
            {
                return res;
            }

            // Ist eine von den gesuchten Versionen?
            if ( version.find("00.04.00.00") != std::string::npos || version.find("00.04.00.01") != std::string::npos )
            {
                // Auf entsprechende Funktion verzweigen
                return Module_hardware::read_display_type( device_address, val, true );
            }
        }
    }

    return Module_hardware::read_keypad_type( device_address, val, true );
}

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und stellt einen Bugfix da.
*
*/
int Datafox::Communication::Module_system::read_transponder_reader_type( uint8_t device_address, uint8_t &val, bool checked )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    int res;
    std::string version;
    // Befindet sich im Bootloader?
    if ( !checked && (res = read_firmware_version( device_address, version )) != IV_ERROR_SUCCESS )
    {
        if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
        {
            // Befehl wird nicht unterstuetzt
            // Geraet befindet sich im Bootloadermodus
            if ( (res = read_bootloader_version( device_address, version )) != IV_ERROR_SUCCESS )
            {
                return res;
            }

            // Ist eine von den gesuchten Versionen?
            if ( version.find("00.04.00.00") != std::string::npos || version.find("00.04.00.01") != std::string::npos )
            {
                // Auf entsprechende Funktion verzweigen
                return Module_hardware::read_keypad_type( device_address, val, true );
            }
        }
    }

    return Module_hardware::read_transponder_reader_type( device_address, val, true );
}

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und ist aufgrund eines Bugfixes hier implementiert.
*
*/
int Datafox::Communication::Module_system::read_hardware_info( uint8_t device_address, StructHardwareInfo &hardware_info )
{
    int res = IV_ERROR_SUCCESS;
    bool readOneByOne = true;

    // Struktur initialisieren
    memset( &hardware_info, 0xFF, sizeof(StructHardwareInfo) );
    // Administrationsflags ruecksetzen
    hardware_info.administrationFlags = 0;

    if ( (res = Module_hardware::read_hardware_info( device_address, hardware_info )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        // Oder die erwartetete Datenstruktur passt nicht.
        if ( !(((res >> 8) & 0xFF) == 2) && !(((res >> 8) & 0xFF) == 3) && !(((res >> 8) & 0xFF) == 4) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Daten bereits erfolgreich ermittelt, nun nicht mehr eines nach dem anderen lesen.
        readOneByOne = false;
        // Bei Setupoptionen nur die der Klasse bekannten weiterreichen.
        hardware_info.optionsSetup &= IV_VALUE_SETUP_OPTION_MASK;
        // Bei den Bezahloptionen nur die der Klasse bekannten weiterreichen.
        if ( hardware_info.optionsPayable == 0xFF )
        {
            // Wert nicht initialisiert.
            hardware_info.optionsPayable = 0;
        }
        else
        {
            // BOt 2016-07-26
            // Bezahloptions müssen aufgrund fehlerhaft gelieferter Standardwerte separat nachgelesen werden.
            uint8_t po_mask = IV_VALUE_PAYABLE_OPTION_ALL;
            if ( read_buy_options( device_address, po_mask, hardware_info.optionsPayable ) != IV_ERROR_SUCCESS )
                hardware_info.optionsPayable = 0;
            else
                hardware_info.optionsPayable &= po_mask;
        }
    }

    //#####################################################
    // Version der Hardwareinfo ermitteln
    if ( readOneByOne && (res = read_hardware_info_version( device_address, hardware_info.version )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Liegt keine Hardwareinfo vor?
    if ( hardware_info.version == 0xFF )
        hardware_info.version = 0;

    // Geraetetyp ermitteln
    if ( readOneByOne && (res = read_device_type( device_address, hardware_info.typeDevice )) != IV_ERROR_SUCCESS )
    {
        // Wenn der Modulfehler -Funktion unbekannt- ist,
        // dann liegt eine FW 04.01.01.<k vor.
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hardware_info.typeDevice == 0xFF )
        hardware_info.typeDevice = IV_VALUE_TYPE_DEVICE_PZE;

    // Boardtyp ermitteln
    if ( readOneByOne && (res = read_board_type( device_address, hardware_info.typeBoard )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeBoard == 0xFF )
        hardware_info.typeBoard = IV_VALUE_TYPE_BOARD_1_4;

    // RS485 ermitteln
    if ( readOneByOne && (res = read_rs485_communication_component( device_address, hardware_info.componentRS485_0 )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hardware_info.componentRS485_0 == 0xFF )
        hardware_info.componentRS485_0 = 0;

    // TCPIP ermitteln
    if ( readOneByOne && (res = read_network_adapter_type( device_address, hardware_info.typeTCPIP )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hardware_info.typeTCPIP == 0xFF )
        hardware_info.typeTCPIP = IV_VALUE_TYPE_TCPIP_NONE;

    // ModemMC35i ermitteln
    if ( readOneByOne && (res = read_modem_type( device_address, hardware_info.typeModemMC )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hardware_info.typeModemMC == 0xFF )
        hardware_info.typeModemMC = IV_VALUE_TYPE_MODEM_MC_NONE;

    // RS485 (intern) ermitteln
    if ( readOneByOne && (res = read_rs485_access_control_component( device_address, hardware_info.componentRS485_1 )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hardware_info.componentRS485_1 == 0xFF )
        hardware_info.componentRS485_1 = 0;

    // Typ ermitteln (erst ab 04.01.01.p / Bootloader 4.0)
    if ( readOneByOne && (res = read_display_type( device_address, hardware_info.typeDisplay )) != IV_ERROR_SUCCESS )
    {
        // Wenn der Modulfehler -Funktion unbekannt- ist,
        // dann liegt eine FW 04.01.01.<p vor.
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeDisplay == 0xFF )
        hardware_info.typeDisplay = IV_VALUE_TYPE_DISPLAY_SMALL;

    // Typ ermitteln
    if ( readOneByOne && (res = read_keypad_type(device_address, hardware_info.typeKeypad)) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeKeypad == 0xFF )
        hardware_info.typeKeypad = IV_VALUE_TYPE_KEYPAD_9;

    // Typ ermitteln
    if ( readOneByOne && (res = read_transponder_reader_type( device_address, hardware_info.typeTransponder )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeTransponder == 0xFF )
        hardware_info.typeTransponder = IV_VALUE_TYPE_TRANSPONDER_TSR32;

    // Typ ermitteln
    if ( readOneByOne && (res = read_mmc_type( device_address, hardware_info.typeMMC )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeMMC == 0xFF )
        hardware_info.typeMMC = IV_VALUE_TYPE_MMC_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = read_radio_type( device_address, hardware_info.typeRadio )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeRadio == 0xFF )
        hardware_info.typeRadio = IV_VALUE_TYPE_RADIO_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = read_gps_type( device_address, hardware_info.typeGPS )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeGPS == 0xFF )
        hardware_info.typeGPS = IV_VALUE_TYPE_GPS_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = read_usb_component( device_address, hardware_info.componentUSB )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.componentUSB == 0xFF )
        hardware_info.componentUSB = 0;

    // Typ ermitteln
    if ( readOneByOne && (res = read_DCF77_type( device_address, hardware_info.typeDCF77 )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeDCF77 == 0xFF )
        hardware_info.typeDCF77 = IV_VALUE_TYPE_DCF77_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = read_analog_input_component( device_address, hardware_info.componentInAnalog )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.componentInAnalog == 0xFFFFFFFF )
        hardware_info.componentInAnalog = IV_VALUE_COMPONENT_IN_ANALOG_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = read_digital_input_component( device_address, hardware_info.componentInDigital )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.componentInDigital == 0xFFFFFFFF )
        hardware_info.componentInDigital = IV_VALUE_COMPONENT_IN_DIGITAL_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = read_digital_output_component( device_address, hardware_info.componentOutDigital )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.componentOutDigital == 0xFFFFFFFF )
        hardware_info.componentOutDigital = IV_VALUE_COMPONENT_OUT_DIGITAL_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = read_fingerprint_scanner_type( device_address, hardware_info.typeFingerprint )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hardware_info.typeFingerprint == 0xFF )
        hardware_info.typeFingerprint = IV_VALUE_TYPE_FINGERPRINT_NONE;

    if ( readOneByOne )
    {
        hardware_info.typeExternalReader = 0xFF;
        // Typ ermitteln
        if ( (res = read_external_transponder_reader_type( device_address, hardware_info.typeExternalReader )) != IV_ERROR_SUCCESS )
        {
            // Nicht der Modulfehler -Funktion unbekannt-
            if ( !(((res >> 8) & 0xFF) == 2) )
                return res;

            // Fehlerobjekt zurueck setzen
            res = IV_ERROR_SUCCESS;
        }
    }

    // Wenn ZK Option vorhanden
    if ( hardware_info.componentRS485_1 )
    {
        // aber der Externe Lesertyp undefiniert ist,
        // dann wird von TS-Lesern ausgegangen.
        if ( hardware_info.typeExternalReader == 0xFF )
            hardware_info.typeExternalReader = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
    }
    else
    {
        // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
        // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
        // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
        // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
        // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
        // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
        // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
        // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
        // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
        // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
        // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
        // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
        // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
        // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
        // IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL und Board 1.0
        // IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER und Board 2.1
        // IV_VALUE_TYPE_DEVICE_TERMINAL_3_5 und Board 1.4
        // IV_VALUE_TYPE_DEVICE_ZK-Leser plus und Board 2.1
        // IV_VALUE_TYPE_DEVICE_ZK-Zentrale und Board 1.0
        bool evoLine = hardware_info.typeDevice == 0x0a 
            || hardware_info.typeDevice == 0x0b 
            || hardware_info.typeDevice == 0x0c 
            || hardware_info.typeDevice == 0x0d 
            || hardware_info.typeDevice == 0x0e
            || hardware_info.typeDevice == 0x0F
            || hardware_info.typeDevice == 0x10
            || hardware_info.typeDevice == 0x11
            || hardware_info.typeDevice == 0x12
            || hardware_info.typeDevice == 0x13
            || hardware_info.typeDevice == 0x14
            || hardware_info.typeDevice == 0x15
            || hardware_info.typeDevice == 0x16
            || hardware_info.typeDevice == 0x17
            || hardware_info.typeDevice == 0x18
            || hardware_info.typeDevice == 0x19;
        if ( !evoLine && hardware_info.typeBoard < 0x40 )
        {
            // Die Abhängigkeit dieser Option von hwInfo.componentRS485_1 ist nur bei den älteren Geräten vorhanden.
            // Keine Externen leser
            hardware_info.typeExternalReader = IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED;
        }
    }
    // BUGFIX
    // Eine Firmware die die Serien TS und Protect unterstützt setzt als unterstützte Serien
    // TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
    // so dass der Wert der Hardwareinfo eindeutig ist.
    // Werden beide Flags vorgefunden wird von TS ausgegangen,
    // wird Protect alleine vorgefunden wird von PHG alleine ausgegangen.
    if ( hardware_info.typeExternalReader & 0x01 && hardware_info.typeExternalReader & 0x04 )
        hardware_info.typeExternalReader = 0x01;
    else if ( hardware_info.typeExternalReader & 0x04 )
        // BUGFIX
        // Eine Firmware < 04.01.04.43 stellt fest ihre unterstuetzte Serie und verwendet 4 fuer PHG.
        hardware_info.typeExternalReader = 0x02;

    // Option ermitteln
    if ( readOneByOne && (res = read_io_module_option( device_address, hardware_info.optionIOModule )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.optionIOModule == 0xFF )
        hardware_info.optionIOModule = IV_VALUE_OPTION_IOMODULE_NONE;

    // Option ermitteln
    if ( readOneByOne && (res = read_timeboy_adaption_option( device_address, hardware_info.optionTBAdaption )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist und ein Mobil-MasterIV vorliegt, dann aktiv ansonsten deaktiviert.
    if ( hardware_info.optionTBAdaption == 0xFF )
    {
        if ( hardware_info.typeDevice == IV_VALUE_TYPE_DEVICE_MOBIL )
            hardware_info.optionTBAdaption = IV_VALUE_OPTION_TBADAPTION;
        else
            hardware_info.optionTBAdaption = IV_VALUE_OPTION_TBADAPTION_NONE;
    }

    // Option ermitteln
    if ( readOneByOne && (res = read_com_server_option( device_address, hardware_info.optionComserver )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // BUGFIX:
        // Dieser Befehl existiert bereits in der Firmware obwohl die Option noch nicht unterstützt wird.
        // Er gehört zur Gruppe 2 der Zusatzoptionen liegt aber per Befehl in der Gruppe 1
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.optionComserver == 0xFF )
        hardware_info.optionComserver = IV_VALUE_OPTION_COMSERVER_NONE;

    // Option ermitteln
    if ( readOneByOne && (res = read_transponder_write_option( device_address, hardware_info.optionTPWrite )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Fasthack um festzustellen, dass die Optionen der Gruppe 2 verfuegbar
        hardware_info.administrationFlags |= HIAF_HAS_OPTIONS2;
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.optionTPWrite == 0xFF )
        hardware_info.optionTPWrite = IV_VALUE_OPTION_TPWRITE_NONE;

    // Option ermitteln
    if ( readOneByOne && (res = read_http_option( device_address, hardware_info.optionHttp )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von aktiviert ausgegangen.
    if ( hardware_info.optionHttp == 0xFF )
    {
        hardware_info.optionHttp = IV_VALUE_OPTION_HTTP_NONE;

        if ( hardware_info.typeModemMC )
            hardware_info.optionHttp = IV_VALUE_OPTION_HTTP;
    }

    // Option ermitteln
    if ( readOneByOne && (res = read_localization_option( device_address, hardware_info.optionLanguage )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von aktiviert ausgegangen.
    if ( hardware_info.optionLanguage == 0xFF )
        hardware_info.optionLanguage = IV_VALUE_OPTION_LANGUAGE;


    // Option ermitteln
    if ( readOneByOne && (res = read_fingerprint_scanner_update_option( device_address, hardware_info.optionBiokeyUpdate )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.optionBiokeyUpdate == 0xFF )
        hardware_info.optionBiokeyUpdate = IV_VALUE_OPTION_BIOKEY_UPDATE_NONE;

    uint16_t mask = 0xFFFF;
    // Option ermitteln
    if ( readOneByOne && (res = read_setup_options( device_address, mask, hardware_info.optionsSetup )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Fasthack um festzustellen, dass die Optionen der Gruppe 3 verfuegbar
        hardware_info.administrationFlags |= HIAF_HAS_OPTIONS3;
    }

    // Wenn die Option undefiniert ist dann wird von maximal moeglich ausgegangen.
    //	if (hwInfo.optionsSetup == 0xFFFF) {
    //#ifdef _DEBUG
    hardware_info.optionsSetup = (mask & hardware_info.optionsSetup);
    //#endif
    //	}

    // Typ ermitteln
    if ( readOneByOne && (res = read_touchscreen_type( device_address, hardware_info.typeTouch )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        hardware_info.administrationFlags |= HIAF_HAS_OPTIONS4;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.typeTouch == 0xFF )
        hardware_info.typeTouch = IV_VALUE_TYPE_TOUCH_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = read_sensor_type( device_address, hardware_info.typeSensor )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.typeSensor == 0xFF )
        hardware_info.typeSensor = IV_VALUE_TYPE_SENSOR_NONE;

    // Komponente ermitteln
    if ( readOneByOne && (res = read_battery_component( device_address, hardware_info.componentBattery )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.componentBattery == 0xFF )
        hardware_info.componentBattery = 0;

    // Komponente ermitteln
    if ( readOneByOne && (res = read_battery_charger_component( device_address, hardware_info.componentCharge )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hardware_info.componentCharge == 0xFF )
        hardware_info.componentCharge = 0;

    // Options ermitteln
    uint8_t po_mask = IV_VALUE_PAYABLE_OPTION_ALL;
    if ( readOneByOne && (res = read_buy_options( device_address, po_mask, hardware_info.optionsPayable )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Fasthack um festzustellen, dass die Optionen der Gruppe 5 verfuegbar
        hardware_info.administrationFlags |= HIAF_HAS_OPTIONS5;
    }

    return res;
}

// ##################################################################
/**
* @brief Löst die Hardwareinfo in seine String-Representation auf.
*
*/
int Datafox::Communication::Module_system::hardware_info_to_string( StructHardwareInfo hardware_info, std::string &info )
{
    std::string tmp;
    uint32_t bit;
    int res = 1;

    tmp.clear();

    info.append( "Hardwareinformation\r\n" );
    info.append( "------------------------------------------\r\n" );
    info.append( "\r\nBasis\r\n----------\r\n" );
    info.append( "Geraet: " );
    // Moegliche Geraetetypen
    switch ( hardware_info.typeDevice ) 
    {
    case IV_VALUE_TYPE_DEVICE_PZE:                         tmp.append( IV_VALUE_TYPE_DEVICE_PZE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK:                          tmp.append( IV_VALUE_TYPE_DEVICE_ZK_NAME );  break;
    case IV_VALUE_TYPE_DEVICE_BDE:                         tmp.append( IV_VALUE_TYPE_DEVICE_BDE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MOBIL:                       tmp.append( IV_VALUE_TYPE_DEVICE_MOBIL_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MDE:                         tmp.append( IV_VALUE_TYPE_DEVICE_MDE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_AE:                          tmp.append( IV_VALUE_TYPE_DEVICE_AE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TIMEBOY:                     tmp.append( IV_VALUE_TYPE_DEVICE_TIMEBOY_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_EXKLUSIV:                    tmp.append( IV_VALUE_TYPE_DEVICE_EXKLUSIV_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_FLEX:                        tmp.append( IV_VALUE_TYPE_DEVICE_FLEX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE:           tmp.append( IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TERMINAL_2_8:                tmp.append( IV_VALUE_TYPE_DEVICE_TERMINAL_2_8_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TERMINAL_3_5:                tmp.append( IV_VALUE_TYPE_DEVICE_TERMINAL_3_5_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TERMINAL_4_3:                tmp.append( IV_VALUE_TYPE_DEVICE_TERMINAL_4_3_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7:     tmp.append( IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10:      tmp.append( IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7:            tmp.append( IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IO_BOX:                      tmp.append( IV_VALUE_TYPE_DEVICE_IO_BOX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_BOX:                      tmp.append( IV_VALUE_TYPE_DEVICE_ZK_BOX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MOBIL_BOX:                   tmp.append( IV_VALUE_TYPE_DEVICE_MOBIL_BOX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK:              tmp.append( IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK:                tmp.append( IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_KNOTEN:                   tmp.append( IV_VALUE_TYPE_DEVICE_ZK_KNOTEN_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL:           tmp.append( IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER:             tmp.append( IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_READER_PLUS:              tmp.append( IV_VALUE_TYPE_DEVICE_ZK_READER_PLUS_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_ZENTRALE:                 tmp.append( IV_VALUE_TYPE_DEVICE_ZK_ZENTRALE_NAME ); break;
    default:                                               tmp.append( "Undefiniert" ); break;
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    info.append( "Platinenversion: " );
    // Boardtyp pruefen
    switch ( hardware_info.typeBoard )
    {
    case IV_VALUE_TYPE_BOARD_1_0: tmp.append( "1.0" ); break;
    case IV_VALUE_TYPE_BOARD_1_1: tmp.append( "1.1" ); break;
    case IV_VALUE_TYPE_BOARD_1_2: tmp.append( "1.2" ); break;
    case IV_VALUE_TYPE_BOARD_1_3: tmp.append( "1.3" ); break;
    case IV_VALUE_TYPE_BOARD_1_4: tmp.append( "1.4" ); break;
    case IV_VALUE_TYPE_BOARD_1_5: tmp.append( "1.5" ); break;
    case IV_VALUE_TYPE_BOARD_1_6: tmp.append( "1.6" ); break;
    case IV_VALUE_TYPE_BOARD_1_7: tmp.append( "1.7" ); break;
    case IV_VALUE_TYPE_BOARD_1_8: tmp.append( "1.8" ); break;
    case IV_VALUE_TYPE_BOARD_2_0: tmp.append( "2.0" ); break;
    case IV_VALUE_TYPE_BOARD_2_1: tmp.append( "2.1" ); break;
    case IV_VALUE_TYPE_BOARD_2_2: tmp.append( "2.2" ); break;
    case IV_VALUE_TYPE_BOARD_2_3: tmp.append( "2.3" ); break;
    case IV_VALUE_TYPE_BOARD_2_4: tmp.append( "2.4" ); break;
    case IV_VALUE_TYPE_BOARD_2_5: tmp.append( "2.5" ); break;
    case IV_VALUE_TYPE_BOARD_2_6: tmp.append( "2.6" ); break;
    case IV_VALUE_TYPE_BOARD_2_7: tmp.append( "2.7" ); break;
    case IV_VALUE_TYPE_BOARD_3_0: tmp.append( "3.0" ); break;
    case IV_VALUE_TYPE_BOARD_3_1: tmp.append( "3.1" ); break;
    case IV_VALUE_TYPE_BOARD_3_4: tmp.append( "3.4" ); break;
    case IV_VALUE_TYPE_BOARD_4_0: tmp.append( "4.0" ); break;
    case IV_VALUE_TYPE_BOARD_4_1: tmp.append( "4.1" ); break;
    case IV_VALUE_TYPE_BOARD_4_2: tmp.append( "4.2" ); break;
    case IV_VALUE_TYPE_BOARD_4_3: tmp.append( "4.3" ); break;
    case IV_VALUE_TYPE_BOARD_4_4: tmp.append( "4.4" ); break;
    case IV_VALUE_TYPE_BOARD_4_5: tmp.append( "4.5" ); break;
    case IV_VALUE_TYPE_BOARD_4_6: tmp.append( "4.6" ); break;
	case IV_VALUE_TYPE_BOARD_4_7: tmp.append("4.7"); break;
    case IV_VALUE_TYPE_BOARD_4_8: tmp.append( "4.8" ); break;

	default:                      tmp.append( "Undefiniert" ); break;
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    /////////////////////////////////////////////
    // Hauptkommunikationen
    info.append( "\r\nHauptkommunikationen\r\n------------------------------\r\n" );

    // Funk pruefen
    info.append( "Funk: " );
    if ( hardware_info.typeRadio )
    {
        switch ( hardware_info.typeRadio )
        {
        case IV_VALUE_TYPE_RADIO_WRAPTHOR: tmp.append( "Bluetooth" ); break;
        case IV_VALUE_TYPE_RADIO_ZIGBEE:   tmp.append( "Zigbee" ); break;
        default:                           tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // ModemMCxx pruefen
    info.append( "Mobilfunk: " );
    if ( hardware_info.typeModemMC )
    {
        switch ( hardware_info.typeModemMC )
        {
        case IV_VALUE_TYPE_MODEM_MC_39: tmp.append( "Siemens MC39" ); break;
        case IV_VALUE_TYPE_MODEM_MC_55: tmp.append( "Siemens MC55" ); break;
        case IV_VALUE_TYPE_MODEM_RS9110_N_11_24: tmp.append( "Redpine RS9110-N-11-24" ); break;
        default: tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // RS485 pruefen
    tmp = boost::str( boost::format( "RS485: %1%\r\n" ) % (hardware_info.componentRS485_0 ? "Ja" : "Nein" ) );
    info.append( tmp );
    tmp.clear();

    // TCPIP pruefen
    info.append( "TCPIP: " );
    if ( hardware_info.typeTCPIP )
    {
        switch ( hardware_info.typeTCPIP )
        {
        case IV_VALUE_TYPE_TCPIP_XPORT:     tmp.append( "Ja"); break;
        case IV_VALUE_TYPE_TCPIP_MATCHPORT: tmp.append( "Ja, über WLAN"); break;
        default:                            tmp.append( "Undefiniert"); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // USB pruefen
    tmp = boost::str( boost::format( "USB: %1%\r\n" ) % (hardware_info.componentUSB ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    /////////////////////////////////////////////
    // Zusatzoptionen
    info.append( "\r\nWaehlbare Optionen der Firmware\r\n------------------------------\r\n");

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "Anbindung IO-Modul: %1%\r\n" ) % (hardware_info.optionIOModule ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "Anbindung TimeboyIII: %1%\r\n" ) % (hardware_info.optionTBAdaption ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "Com-Servermodus: %1%\r\n" ) % (hardware_info.optionComserver ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "HTTP-Protokoll: %1%\r\n" ) % (hardware_info.optionHttp ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "Transponder ueber Transpondermenue schreiben: %1%\r\n" ) % (hardware_info.optionTPWrite ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "Textdaten benötigt: %1%\r\n" ) % (hardware_info.optionLanguage ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzoption eingestellt?
    tmp = boost::str( boost::format( "Biokeymodulupdate möglich: %1%\r\n" ) % (hardware_info.optionBiokeyUpdate ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Setupoptionen (intern) ermitteln
    tmp = boost::str( boost::format( "Setupoptionen (0x%04X): " ) % hardware_info.optionsSetup );
    info.append( tmp );
    tmp.clear();
    for ( bit = 1; ; bit <<= 1 )
    {
        // Nach maximal bekannter Option abbrechen
        if ( !(bit & IV_VALUE_SETUP_OPTION_MASK) )
            break;

        if ( tmp.size() ) 
            tmp.append( ", ");

        switch ( bit )
        {
        case IV_VALUE_SETUP_OPTION_IOCONTROL:
            tmp.append( "Signalverarbeitung( " ); 
            if ( bit & hardware_info.optionsSetup )
                tmp.append( "Ja )" );
            else
                tmp.append( "Nein )" );

            break;

        default:
            tmp.append( "Undefiniert" );
            break;
        }
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // Bezahlbare Optionen ermitteln
    tmp = boost::str( boost::format( "Erweiterte Optionen (0x%02X): ") % hardware_info.optionsPayable );
    info.append( tmp );
    tmp.clear();
    for ( bit = 1; ; bit <<= 1 )
    {
        // Nach maximal bekannter Option abbrechen
        if ( !(bit & IV_VALUE_PAYABLE_OPTION_MASK) )
            break;

        if ( tmp.size() ) 
            tmp.append( ", " );

        switch ( bit )
        {
        case IV_VALUE_PAYABLE_OPTION_DATA_ON_CARD:
            tmp.append( "Data on Card( " ); 
            if ( bit & hardware_info.optionsPayable )
                tmp.append( "Ja )");
            else
                tmp.append( "Nein )");

            break;
        default:
            tmp.append( "Undefiniert" );
            break;
        }
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    /////////////////////////////////////////////
    // Zusatzbaugruppen
    info.append( "\r\nZusatzbaugruppen\r\n------------------------------\r\n");
    tmp.clear();

    // Display pruefen
    info.append( "Anzeige: " );
    if ( hardware_info.typeDisplay )
    {
        switch ( hardware_info.typeDisplay )
        {
        case IV_VALUE_TYPE_DISPLAY_SMALL:         tmp.append( "4 Zoll Monochrome" ); break;
        case IV_VALUE_TYPE_DISPLAY_BIG:           tmp.append( "6 Zoll Monochrome" ); break;
        case IV_VALUE_TYPE_DISPLAY_SMALLER:       tmp.append( "1,9 Zoll Color" ); break;
        case IV_VALUE_TYPE_DISPLAY_TIMEBOY_2ZOLL: tmp.append( "2 Zoll Color" ); break;
        default:                                  tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // Touch pruefen
    info.append( "Touch: " );
    if ( hardware_info.typeTouch )
    {
        switch ( hardware_info.typeTouch )
        {
        case IV_VALUE_TYPE_TOUCH_SMALL:   tmp.append( "4 Zoll" ); break;
        case IV_VALUE_TYPE_TOUCH_BIG:     tmp.append( "6 Zoll" ); break;
        default:                          tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // Sensor pruefen
    info.append( "Sensor: " );
    if ( hardware_info.typeSensor )
    {
        switch ( hardware_info.typeSensor )
        {
        case IV_VALUE_TYPE_SENSOR_VIBRATION:   tmp.append( "Vibration" ); break;
        case IV_VALUE_TYPE_SENSOR_SPEEDUP:     tmp.append( "Beschleunigung" ); break;
        default:                               tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();


    // DigitalOut pruefen
    info.append( "Ausgaenge Digital: " );
    if ( hardware_info.componentOutDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( bit & hardware_info.componentOutDigital )
            {
                if ( tmp.size() )
                    tmp.append( ", ");

                switch ( bit )
                { // Digitale Ausgaenge
                case 0x01: tmp.append( "dO1" ); break;
                case 0x02: tmp.append( "dO2" ); break;
                case 0x04: tmp.append( "dO3" ); break;
                case 0x08: tmp.append( "dO4" ); break;
                default:   tmp.append( "Undefiniert" ); break;
                }
            }

            if ( bit == 0x8000000 )
                break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // AnalogIn pruefen
    info.append( "Eingaenge Analog: " );
    if ( hardware_info.componentInAnalog )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( bit & hardware_info.componentInAnalog )
            {
                if ( tmp.size() )
                    tmp.append( ", ");

                switch ( bit )
                { // Analoge Eingaenge
                case 0x01: tmp.append( "aI1" ); break;
                case 0x02: tmp.append( "aI2" ); break;
                default:   tmp.append( "Undefiniert" ); break;
                }
            }

            if ( bit == 0x8000000 )
                break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // DigitalIn pruefen
    info.append( "Eingaenge Digital: " );
    if ( hardware_info.componentInDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( bit & hardware_info.componentInDigital )
            {
                if ( tmp.size() )
                    tmp.append( ", " );

                switch ( bit )
                { // Digitale Eingaenge
                case 0x01: tmp.append( "dI1" ); break;
                case 0x02: tmp.append( "dI2" ); break;
                case 0x04: tmp.append( "dI3" ); break;
                case 0x08: tmp.append( "dI4" ); break;
                case 0x10: tmp.append( "dI5" ); break;
                case 0x20: tmp.append( "dI6" ); break;
                case 0x40: tmp.append( "dI7" ); break;
                case 0x80: tmp.append( "dI8" ); break;
                default:   tmp.append( "Undefiniert" ); break;
                }
            }

            if ( bit == 0x8000000 )
                break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // Zusatzkomponente bestückt?
    tmp = boost::str( boost::format( "Batterie bestückt: %1%\r\n") % (hardware_info.componentBattery ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Zusatzkomponente bestückt?
    tmp = boost::str( boost::format( "Ladeschaltung bestückt: %1%\r\n") % (hardware_info.componentCharge ? "Ja" : "Nein") );
    info.append( tmp );
    tmp.clear();

    // Fingerprint pruefen
    info.append( "Fingerprint: " );
    if ( hardware_info.typeFingerprint )
    {
        switch ( hardware_info.typeFingerprint )
        {
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_3000: tmp.append( "Ja (3000)" ); break;
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4000: tmp.append( "Ja (4000)" ); break;
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4020: tmp.append( "Ja (4020)" ); break;
        default:                                    tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // DCF77 pruefen
    info.append( "Funkuhr/Barcode (intern): " );
    if ( hardware_info.typeDCF77 )
    {
        switch ( hardware_info.typeDCF77 )
        {
        case IV_VALUE_TYPE_DCF77_HKW:            tmp.append( "Funkuhr" ); break;
        case IV_VALUE_TYPE_DCF77_BARCODE:        tmp.append( "Barcode Typ 1" ); break;
        case IV_VALUE_TYPE_DCF77_BARCODE_ZEBEX:  tmp.append( "Barcode Typ 2" ); break;
        default:                                 tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // GPS pruefen
    info.append( "GPS: " );
    if ( hardware_info.typeGPS )
    {
        switch ( hardware_info.typeGPS )
        {
        case IV_VALUE_TYPE_GPS_GN80:       tmp.append( "Ja, Typ 1" ); break;
        case IV_VALUE_TYPE_GPS_5H:         tmp.append( "Ja, Typ 2" ); break;
        case IV_VALUE_TYPE_GPS_MOUSE_5H:   tmp.append( "Ja, Typ 3" ); break;
        default:                           tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // MMC pruefen
    info.append( "MM Card: " );
    if ( hardware_info.typeMMC )
    {
        switch( hardware_info.typeMMC )
        {
        case IV_VALUE_TYPE_MMC_EXTENSION: tmp.append( "Speichererweiterung" ); break;
        case IV_VALUE_TYPE_MMC_DISK:      tmp.append( "Wechseldatenträger" ); break;
        case IV_VALUE_TYPE_MMC_USB:       tmp.append( "USB-Host Wechseldatenträger" ); break;
        default:                          tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // Tastatur pruefen
    info.append( "Tastatur: " );
    if ( hardware_info.typeKeypad )
    {
        switch ( hardware_info.typeKeypad )
        {
        case IV_VALUE_TYPE_KEYPAD_9:  tmp.append( "9 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_58: tmp.append( "58 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_29: tmp.append( "29 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_7:  tmp.append( "7 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_25: tmp.append( "25 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_20: tmp.append( "20 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_3:  tmp.append( "3 Tasten" ); break;
        default:                      tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // Transponder pruefen
    info.append( "Transponderleser: " );
    if ( hardware_info.typeTransponder )
    {
        switch ( hardware_info.typeTransponder )
        {
        case IV_VALUE_TYPE_TRANSPONDER_TSR30:          tmp.append( "[EM4102, Hitag2]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TSR32:          tmp.append( "[EM4102, EM4450, Hitag]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_PRIMO100:       tmp.append( "[Legic]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_MIFARE:         tmp.append( "[Mifare]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_EXTERN: tmp.append( "[IButton (Extern)]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS:    tmp.append( "[SimonsVoss SmartRelais]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_NEDAP:          tmp.append( "[Nedap]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_INTERN: tmp.append( "[IButton (Intern)]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_HID_PROXKEY:    tmp.append( "[HID ProxKey]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TWN3_MULTI_ISO: tmp.append( "[Mifare Desfire]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_PRIMO130:       tmp.append( "[Legic Advant]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TSR21:          tmp.append( "[EM4102, EM4450, Hitag]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_HID_ICLASS:     tmp.append( "[HID iClass, Wiegand, Clock Data]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TB_MIFARE:      tmp.append( "[Timeboy Mifare]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TB_LEGIC:       tmp.append( "[Timeboy Legic]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS_G2: tmp.append( "[SimonsVoss SmartRelais G2]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TWN3_MIFARE_NFC:tmp.append( "[Mifare NFC (ISO14443)]" ); break;

        default:                                       tmp.append( "[Undefiniert]" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // RS485 (intern) ermitteln
    info.append( "Zutrittskontrolle: " );
    if ( hardware_info.componentRS485_1 )
    {
        switch ( hardware_info.typeExternalReader )
        {
        case IV_VALUE_TYPE_EXTERNAL_READER_GIS:     tmp.append( "TS-Serie" ); break;
        case IV_VALUE_TYPE_EXTERNAL_READER_PHG:     tmp.append( "PHG-Serie" ); break;
        case IV_VALUE_TYPE_EXTERNAL_READER_PROTECT: tmp.append( "Protect-Serie" ); break;
        case IV_VALUE_TYPE_EXTERNAL_READER_IDTRONIC: tmp.append( "ID Tronic-Serie" ); break;
        case 0xFF:                                  tmp.append( "Unbekannt" ); break;
        default:                                    tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    return res;
}

//#########################################################
//####################### SYSTEM ##########################
//#########################################################

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::remote_call( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count )
{
    if ( module == IV_MODULE_SYSTEM )
    {
        switch ( func )
        {
            // Ermitteln der Firmwareversion
        case IV_FUNCTION_GET_VERSION:
            {
                int res = IV_ERROR_SUCCESS;
                std::string version;
                if ((res = read_firmware_version(get_device_address_sender(), version)) == IV_ERROR_SUCCESS)
                {
                    version.copy(reinterpret_cast<char *>(data), version.size());
                    data[version.size()] = 0;
                    count = static_cast<int>( version.size() ) + 1;
                }

                return res;
            }

            // Setzen/Ermitteln von: SerialNum
        case IV_FUNCTION_SET_SERIAL:
            return write_serial_number(get_device_address_sender(), *(uint32_t*)data, *(uint32_t*)&data[4]);

        case IV_FUNCTION_GET_SERIAL:
            return read_serial_number(get_device_address_sender(), (uint32_t&)data);

            // Setzen/Ermitteln von: DateTime (YYYY-MM-DD HH:MM:SS\0)
        case IV_FUNCTION_SET_RTC:
            return write_date_time(get_device_address_sender(), (char*)data);

        case IV_FUNCTION_GET_RTC:
            return read_date_time(get_device_address_sender(), (char*)data);

            // Setzen/Ermitteln von: Variable (Systemvariablen GPRS) 
        case IV_FUNCTION_SET_SYSTEM_VARIABLE:
            {
                std::string name(reinterpret_cast<char *>(&data[0]));
                std::string val(reinterpret_cast<char *>(&data[name.size() + 1]));
                return write_system_variable(get_device_address_sender(), name, val);                                      
            }

        case IV_FUNCTION_GET_SYSTEM_VARIABLE:
            {
                std::string name(reinterpret_cast<char *>(&data[0]));
                std::string val;
                int res = read_system_variable(get_device_address_sender(), name, val);
                if (res == IV_ERROR_SUCCESS)
                {
                    count += static_cast<int>( val.copy(reinterpret_cast<char *>(&data[name.size() + 1]), std::string::npos) );
                    data[count] = 0; // Nullterminierung
                    ++count;
                }
                return res;
            }
            // Setzen/Ermitteln von: Relaiszustand 
        case IV_FUNCTION_SET_RELAY:
            return 0x0200;

        case IV_FUNCTION_GET_RELAY:
            return 0x0200;

        // Schreiben/Lesen von: Externen Flash
        case IV_FUNCTION_SET_FLASH:
            return write_to_flash( get_device_address_sender(), *reinterpret_cast<uint32_t*>( &data[0] ), &data[4], count - 4 );

        case IV_FUNCTION_GET_FLASH:
            return read_from_flash( get_device_address_sender(), *reinterpret_cast<uint32_t*>( &data[0] ), &data[4], count - 4 );

        // Schreiben/Lesen von: EEPROM
        case IV_FUNCTION_SET_EEPROM:
            return 0x0200;

        case IV_FUNCTION_GET_EEPROM:
            return 0x0200;

        // Schreiben/Lesen von: Program-Flash schreiben/lesen
        case IV_FUNCTION_SET_PROGRAM:
            return 0x0200;

        case IV_FUNCTION_GET_PROGRAM:
            return 0x0200;

            // Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
        case IV_FUNCTION_BEGIN_SET_SEGMENT:
            return enter_write_data_segment( get_device_address_sender(), *(uint16_t *)&data[0],
                    reinterpret_cast<uint32_t&>( data[2] ), *reinterpret_cast<uint32_t*>( &data[6] ),
                    *reinterpret_cast<uint32_t*>( &data[10] ) );

        case IV_FUNCTION_BEGIN_SET_SEGMENT_OK:
            return reply_enter_write_data_segment( get_device_address_sender() );

        case IV_FUNCTION_END_SET_SEGMENT:
            return exit_write_data_segment( get_device_address_sender(), *(uint16_t *)&data[0],
                    *reinterpret_cast<uint32_t*>( &data[2] ) );

        case IV_FUNCTION_END_SET_SEGMENT_OK:
            return reply_exit_write_data_segment( get_device_address_sender() );

            // Ermitteln von: Bootloaderversion
        case IV_FUNCTION_GET_VERSION_BOOTLOADER:
            {
                int res = IV_ERROR_SUCCESS;
                std::string version;
                if ((res = read_bootloader_version(get_device_address_sender(), version)) == IV_ERROR_SUCCESS)
                {
                    version.copy(reinterpret_cast<char *>(data), version.size());
                    data[version.size()] = 0;
                    count = static_cast<int>( version.size() ) + 1;
                }
                return res;
            }

            // Durchfuehren eines Resets der Speicherbereiche
        case IV_FUNCTION_SYSTEM_RESET:
            return 0x0200;

            // Ins Bios wechseln
        case IV_FUNCTION_ENTER_BIOS:
            return enter_bios_mode( get_device_address_sender() );

            // Bioswechsel erfolgt
        case IV_FUNCTION_ENTERED_BIOS:
            return reply_enter_bios_mode( get_device_address_sender() );

            // Bios verlassen (aenderungen uebernehmen
        case IV_FUNCTION_EXIT_BIOS:
            return exit_bios_mode( get_device_address_sender() );

            // Biosverlassen erfolgt
        case IV_FUNCTION_EXITED_BIOS:
            return reply_exit_bios_mode( get_device_address_sender() );

            // Setzt/liest das zu verwendende Interface
        case IV_FUNCTION_SET_INTERFACE:
            return 0x0200;

        case IV_FUNCTION_GET_INTERFACE:
            return 0x0200;

            // Setzt/liest die zu verwendende BaudRate
        case IV_FUNCTION_SET_BAUD:
            return 0x0200;

        case IV_FUNCTION_GET_BAUD:
            return 0x0200;

            // Setzt/liest die zu verwendende DeviceID
        case IV_FUNCTION_SET_DEVICE_ID:
            return 0x0200;

        case IV_FUNCTION_GET_DEVICE_ID:
            return 0x0200;

            // Befehl der beim Geraetestart den Bootloader des ZK - Master IV wiederherstellt.
        case IV_FUNCTION_ENTER_BOOTLOADER:
            return 0x0200;

            // In Monitor Mode von XPort wechselt
            //		case IV_FUNCTION_ENTER_XPORT_MONITOR: return CallEnterXPortMonitor(data, count);
            // Antwort in Monitor Mode gewechselt
        case IV_FUNCTION_ENTER_XPORT_MONITOR_OK:
            return reply_enter_xport_monitor_mode( get_device_address_sender() );

            // Monitorbefehl an XPort und Antwort erhalten.
            //		case IV_FUNCTION_CMD_XPORT: return CallCommandXPort(data, count);

            // In Monitor Mode von XPort wechselt
            //		case IV_FUNCTION_EXIT_XPORT_MONITOR: return CallExitXPortMonitor(data, count);
            // Antwort in Monitor Mode gewechselt
        case IV_FUNCTION_EXIT_XPORT_MONITOR_OK:
            return reply_exit_xport_monitor_mode(get_device_address_sender());

        case IV_FUNCTION_BEGIN_GET_SEGMENT:
            return 0x0200;

        case IV_FUNCTION_BEGIN_GET_SEGMENT_OK:
            return reply_enter_read_data_segment( get_device_address_sender() );

        case IV_FUNCTION_END_GET_SEGMENT:
            return 0x0200;

        case IV_FUNCTION_END_GET_SEGMENT_OK:
            return reply_exit_read_data_segment( get_device_address_sender() );

        case IV_FUNCTION_SYSTEM_RESET_OK:
            return reply_system_reset( get_device_address_sender() );

            //		case IV_FUNCTION_OPEN_COM_SERVER_MODE:  return openComServerMode(GetDeviceID());
            //		case IV_FUNCTION_CLOSE_COM_SERVER_MODE: return closeComServerMode(GetDeviceID()=);
        case IV_FUNCTION_GET_INFO:
        {
            char *name = (char*)data;
            char *param = (char*)&data[strlen(name) + 1];

            //return getInfo( getDeviceIdRemote(), name, param, info, infoLength );
            return 0x0001;
        }


        case IV_FUNCTION_OPEN_COM_SERVER_MODE_OK:
            return reply_enter_com_server_mode( get_device_address_sender() );

            //		case IV_FUNCTION_SET_NOTIFY:
            //		case IV_FUNCTION_GET_NOTIFY:
            //		case IV_FUNCTION_GET_SOFTWARE_INFO:
            //		case IV_FUNCTION_GET_FLASH_PAGE:

        case IV_FUNCTION_GET_INFO_OK:
            return reply_read_module_info( get_device_address_sender() );

        default: return 0x0200; // Funktion nicht definiert
        }
    }

    return Module_hardware::remote_call(module, func, data, count);
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::system_reset( uint8_t device_address, uint16_t modus )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, modus );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        count = sizeof(uint16_t);

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Den Resetmodus uebernehmen.
        *(uint16_t*)m_packet_data = modus;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM , IV_FUNCTION_SYSTEM_RESET, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, m_packet_data, count, IV_PENDING_RESET_TRIALS(get_connection().get_read_timeout()) )) != IV_ERROR_SUCCESS )
        {
            if ( result == IV_ERROR_UNREACHABLE )
            {
                result = IV_ERROR_SUCCESS;
            }
            else
            {
                // Fehler ausgeben
                trace_error( get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, result );
                return result;
            }
        }

		// Neuaushandlung bei verschluesselter Kommunikation
		Aes_crypt_session().reset_session_key();
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }

        // Aktive Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_system_reset( uint8_t device_address )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, device_address );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Leited einen zu übertragenden Datenblock ein. Z. B. Listen.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_write_data_segment( uint8_t device_address, uint16_t art, uint32_t &address, uint32_t size, uint32_t crc )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %i, %i, %i )\n", __FILE__, __LINE__, __func__, device_address, art, address, size, crc );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Meberpuffer aufbereiten
        *(uint16_t*)m_packet_data = art;
        count = sizeof(uint16_t);

        // addresse wegen wohlgeformtem Paket mitgeben
        *(uint32_t*)&m_packet_data[count] = address;
        count += sizeof(uint32_t);

        // geforderte Speichergroesze mitgeben
        *(uint32_t*)&m_packet_data[count] = size;
        count += sizeof(uint32_t);

        // CRC32 fuer Datenblock mitgeben
        *(uint32_t*)&m_packet_data[count] = crc;
        count += sizeof(uint32_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT, result );
            return result;
        }

        // OK

        // Ausgeben der empfangenen Werte

        if ( count == 14 )
        {
            if ( art != *(uint16_t*)m_packet_data )
            {
                trace( "%s(%i):WARNING(1): %s() [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art );
            }

            address = *(uint32_t*)&m_packet_data[sizeof(uint16_t)];

            if ( size != *(uint32_t*)&m_packet_data[sizeof(uint16_t) + sizeof(uint32_t)] )
            {
                trace( "%s(%i):WARNING(1): %s() [ Size ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint32_t*)&m_packet_data[sizeof(uint16_t) + sizeof(uint32_t)], size );
            }

            if ( crc != *(uint32_t*)&m_packet_data[sizeof(uint16_t) + sizeof(uint32_t)*2] )
            {
                trace( "%s(%i):WARNING(1): %s() [ CRC ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint32_t*)&m_packet_data[sizeof(uint16_t) + sizeof(uint32_t)*2], crc );
            }

            trace( "%s(%i):INFO(3): %s() (0x%04X)(0x%08X)(0x%08X)(0x%08X)\n", __FILE__, __LINE__, __func__, art, address, size, crc );
        }
        else 
        {
            trace( "%s(%i):ERROR(1): %s() [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Bei (den einzelnen innenliegenden) Systemstopps,
        // werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
        if ( (art == IV_VALUE_SEGMENT_UPDATE) || (art == IV_VALUE_SEGMENT_UPDATE_BG) || (art == IV_VALUE_SEGMENT_LIST_ACCESS) )
        {
            /*
            short module, func, error = IV_ERROR_SUCCESS;
            // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
            // dieses muss hier seperat abgegriffen werden.
            if ((_result = Pending(device_address, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS) {

            Trace("%s(%i):ERROR(1): %s [ Quittierungsprüfantwort - CRC schlug fehl. ]\n", __FILE__, __LINE__, __func__);
            return _result;
            }

            if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_BEGIN_SET_SEGMENT_OK)) {

            Trace("%s(%i):INFO(3): %s [ Antwort von CRC OK. ]\n", __FILE__, __LINE__, __func__);
            }

            // evntl. aktiven Fehler auswerten
            if (error != IV_ERROR_SUCCESS) {

            _result = error;
            // Fehler ausgeben
            traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
            return _result;
            }

            if (*(uint16_t*)_packetData != art) {

            Trace("%s(%i):WARNING(1): %s [ Die Art der Quittierung ist ungleich. ]\n", __FILE__, __LINE__, __func__);
            } 

            if (count > sizeof(uint16_t)) {

            Trace("%s(%i):WARNING(1): %s [ Datenmenge ist mit (%i) grösser als erwartet. ]\n", __FILE__, __LINE__, __func__, count);
            }
            */
            if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()) )) != IV_ERROR_SUCCESS )
            {
                trace("%s(%i):ERROR(1): %s() [ Quittierungsprüfantwort - CRC schlug fehl. ]\n", __FILE__, __LINE__, __func__);
                return result;
            }

            if (*(uint16_t*)m_packet_data != art)
            {
                trace("%s(%i):WARNING(1): %s() [ Die Art der Quittierung ist ungleich. ]\n", __FILE__, __LINE__, __func__);
            } 
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        std::vector<uint8_t> buffer;
        uint16_t count = 0;
        bool send_active_answer = false;

        Segment_data segment;
        segment.parameter.type = static_cast<Segment_type>( art );
        segment.parameter.address = 0;
        segment.parameter.size = size;
        segment.parameter.crc = crc;
        buffer.resize( sizeof( Segment_parameter ) );
        std::copy_n( (uint8_t *)&segment.parameter, buffer.size(), &buffer[0] );
        count = buffer.size();

        // Dateiname
        boost::filesystem::path path_cache( get_path_cache() );
        path_cache /= boost::uuids::to_string( get_uuid() );

        switch ( segment.parameter.type )
        {
        case Segment_type::Update:
            send_active_answer = true;
            break;
            
        case Segment_type::Update_background:
            send_active_answer = true;
            break;
            
        case Segment_type::Firmware:
            path_cache /= "segment_firmware.bin";
            break;
            
        case Segment_type::Fonts:
            path_cache /= "segment_fonts.bin";
            break;
            
        case Segment_type::Grafics:
            path_cache /= "segment_grafics.bin";
            break;
            
        case Segment_type::Setup:
            path_cache /= "segment_setup.bin";
            break;
            
        case Segment_type::List:
            path_cache /= "segment_list.bin";
            break;
            
        case Segment_type::Setup_extended:
            path_cache /= "segment_setup_extended.bin";
            break;
            
        case Segment_type::List_access:
            path_cache /= "segment_list_access.bin";
            send_active_answer = true;
            break;
            
        case Segment_type::List_timeboy:
            path_cache /= "segment_list_timeboy.bin";
            break;
            
        case Segment_type::Fingerprint:
            path_cache /= "segment_fingerprint.bin";
            break;
            
        case Segment_type::Text:
            path_cache /= "segment_text.bin";
            break;
            
        case Segment_type::Toch_keys:
            path_cache /= "segment_touch_keys.bin";
            break;
            
        case Segment_type::Display_colors:
            path_cache /= "segment_display_colors.bin";
            break;
            
        case Segment_type::Wlan_configs:
            path_cache /= "segment_wlan_configs.bin";
            break;
            
        case Segment_type::Hardware_configs:
            path_cache /= "segment_hardware_configs.bin";
            break;
            
        case Segment_type::File_transfer:
            path_cache /= "segment_file_transfer.bin";
            break;
        }

        // Bei zugeordneter Dateiangabe handelt es sich um ein Datensegment.
        if ( path_cache.extension() == ".bin" )
        {
            segment.file_path = path_cache;
        }

        // Einge ggf. zugewiesene Datei öffnen.
        if ( segment.file_path.size() )
        {
            segment.file_stream = std::make_shared<std::fstream>( segment.file_path.string(), std::ios::out | std::ios::binary );
            if ( segment.file_stream->is_open() )
            {
                segment.file_stream->is_open();
            }
        }

        // Dieses Segment hinzufügen.
        m_segments.push_back( segment );

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT, &buffer[0], count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }

        if ( send_active_answer )
        {
            count = sizeof( uint16_t );
            buffer.resize( count );
            *reinterpret_cast<uint16_t*>( &buffer[0] ) = art;
            // Aktive Antwort des Befehls senden.
            if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, &buffer[0], count )) != IV_ERROR_SUCCESS )
            {
                trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
                return result;
            }
        }
#else
        return 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_enter_write_data_segment( uint8_t device_address )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, device_address );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::exit_write_data_segment(uint8_t device_address, uint16_t art, uint32_t crc)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, art, crc);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        count = sizeof(uint16_t);
        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Meberpuffer aufbereiten
        *(uint16_t*)m_packet_data = art;

        // Beendigung einer Uebertragung eines Datanblocks
        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, result);
            return result;
        }

        // OK
        if (art != *(uint16_t*)m_packet_data)
        {
            trace("%s(%i):ERROR(1): %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art);
        }

        // Bei (den einzelnen innenliegenden) Systemstopps,
        // werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
        /*	
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ((_result = Pending(device_address, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {	
        Trace("%s(%i):ERROR(1): %s [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_END_SET_SEGMENT_OK))
        {			
        Trace("%s(%i):INFO(3): %s [ Antwort von CRC OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK);
        return _result;
        }
        */
        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, result);
            // Flash-Speicher reicht nicht aus.
            if (IV_RESULT_FUNCTION(result) == 1)
                result = -IV_ERROR_NOT_ENOUGH_FLASH;

            return result;
        }

        if (*(uint16_t*)m_packet_data != art)
        {
            trace("%s(%i):ERROR(1): %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art);
        }
        else if (*(uint32_t*)&m_packet_data[sizeof(uint16_t)] != crc)
        {
            trace("%s(%i):ERROR(1): %s [ CRC ist (0x%08X) soll (0x%08X) ]\n", __FILE__, __LINE__, __func__, *(uint32_t*)&m_packet_data[sizeof(uint16_t)], crc);
            return -IV_ERROR_CRC_SEGMENT;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[6];

        // Meberpuffer aufbereiten
        *(uint16_t *)tmp = art;
        count = sizeof( uint16_t );

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }

        *(uint16_t *)&tmp[0] = (uint16_t)m_segments.back().parameter.type;
        count = sizeof( uint16_t );

        *(uint32_t *)&tmp[count] = m_segments.back().parameter.crc;
        count += sizeof( uint32_t );

        // Segment entfernen.
        m_segments.pop_back();

        // Aktive Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        return 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_exit_write_data_segment(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        count = sizeof(uint16_t);
        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Leited einen zu übertragenden Datenblock ein. Z. B. Listen.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_read_data_segment(uint8_t device_address, uint16_t art, uint32_t &address, uint32_t &size, uint32_t &crc)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, art, address, size, crc);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        count = sizeof(uint16_t);

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);


        // Meberpuffer aufbereiten
        *(uint16_t*)m_packet_data = art;

        // addresse wegen wohlgeformtem Paket mitgeben
        *(uint32_t*)&m_packet_data[sizeof(uint16_t)] = address;
        count += sizeof(uint32_t);

        // geforderte Speichergroesze mitgeben
        *(uint32_t*)&m_packet_data[sizeof(uint16_t) + sizeof(uint32_t)] = size;
        count += sizeof(uint32_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT, result);
            return result;
        }

        // OK

        // Ausgeben der empfangenen Werte

        if (count == 10)
        {
            if (art != *(uint16_t*)m_packet_data)
            {
                trace("%s(%i):ERROR(1): %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art);
            }

            address = *(uint32_t*)&m_packet_data[sizeof(uint16_t)];
            size    = *(uint32_t*)&m_packet_data[sizeof(uint16_t) + sizeof(uint32_t)];

            trace("%s(%i):INFO(3): %s [ (0x%04X)(0x%08X)(0x%08X) ]\n", __FILE__, __LINE__, __func__, art, address, size);
        }
        else
        {
            trace("%s(%i):ERROR(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
        /*
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ((_result = Pending(device_address, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {
        Trace("%s(%i):ERROR(1): %s.Pending [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_BEGIN_SET_SEGMENT_OK))
        {
        Trace("%s(%i):INFO(3): %s.Pending [ Antwort von CRC OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
        return _result;
        }
        */
        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT_OK, result);
            return result;
        }

        if (count == (sizeof(uint16_t) + sizeof(uint32_t)))
        {
            if (*(uint16_t*)m_packet_data != art)
            {
                trace("%s(%i):ERROR(1): %s.Pending [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art);
            }

            // CRC uebernehmen
            crc = *(uint32_t*)&m_packet_data[sizeof(uint16_t)];
        }
        else
        {
            trace("%s(%i):ERROR(1): %s.Pending [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[10];
        count = sizeof(uint16_t);

        // Adresse und Size werden zurueckgegeben.

        // Meberpuffer aufbereiten
        *(uint16_t*)tmp = art;

        // addresse wegen wohlgeformtem Paket mitgeben
        *(uint32_t*)&tmp[sizeof(uint16_t)] = address;
        count += sizeof(uint32_t);

        // geforderte Speichergroesze mitgeben
        *(uint32_t*)&tmp[sizeof(uint16_t) + sizeof(uint32_t)] = size;
        count += sizeof(uint32_t);

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_enter_read_data_segment(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::exit_read_data_segment(uint8_t device_address, uint16_t art)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, art);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        count = sizeof(uint16_t);

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);


        // Meberpuffer aufbereiten
        *(uint16_t*)m_packet_data = art;

        // Beendigung einer Uebertragung eines Datanblocks
        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT, result);
            return result;
        }

        // OK
        if (art != *(uint16_t*)m_packet_data)
        {
            trace("%s(%i):ERROR: %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art);
        }

        // Bei (den einzelnen innenliegenden) Systemstopps,
        // werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
        /*
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ((_result = Pending(device_address, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {	
        Trace("%s(%i):ERROR: %s.Pending [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_END_GET_SEGMENT_OK))
        {			
        Trace("%s(%i):INFO(3): %s.Pending [ Antwort von CRC OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK);
        return _result;
        }
        */
        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {

            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK, result);
            return result;
        }

        if (*(uint16_t*)m_packet_data != art)
        {
            trace("%s(%i):ERROR: %s.Pending [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(uint16_t*)m_packet_data, art);
        }

        if (count != sizeof(uint16_t))
        {
            trace("%s(%i):ERROR: %s.Pending [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = sizeof(uint16_t);

        // Meberpuffer aufbereiten
        *(uint16_t*)m_packet_data = art;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_exit_read_data_segment(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

#if IV_PACKET_LENGTH_MAX > 0x300
// Maximale Anzahl Datenbytes fuer setFlash/getFlash. Ctrl-Bytes inkl. weitere Daten wie Modul/Funktion und Addresse 
// ergeben 9 Byte die zusaetzlich abzuziehen sind.
#define IV_FLASH_SIZE_DATA   (IV_PACKET_LENGTH_DATA - (IV_PACKET_LENGTH_CONTROLBLOCK + 9))
#else
// Wegen Bootloader nicht vergroesserbar!!
#define IV_FLASH_SIZE_DATA      512
#endif

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_to_flash(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

#ifdef _DEBUG
    static int loop = 0;
#endif
    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // BlockTransfer durchfuehren.
        controller_segment &controllerSegmentCurrent = get_controller_segment(device_address);

        if (is_stream_transfer())
        {
            m_stream_buffer.append((char*)buffer, count);
        }
        else
        {
            // Bei aktivem Blocktransfer den aktuellen Zeitschlitz halten.
            if (controllerSegmentCurrent.transfer_state == controller_segment::bts_active)
                controllerSegmentCurrent.current_time_slot_time_point = std::chrono::system_clock::now();

            // Gesamten Datenblock aufteilen
            for (addressOffset = 0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
            {
                // Bei aktivem Blocktransfer ggf. unterbrechen.
                if (controllerSegmentCurrent.transfer_state == controller_segment::bts_active 
                    && controllerSegmentCurrent.allowed_transfer_types & controllerSegmentCurrent.transfer_type
                    && controllerSegmentCurrent.duration > std::chrono::seconds(0))
                {
                    if (std::chrono::system_clock::now() > controllerSegmentCurrent.current_time_slot_time_point + controllerSegmentCurrent.duration)
                        return -IV_ERROR_BLOCK_TRANSFER_BREAK;
                }

                // Die Adresse als 1. Parameter eintragen
                *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
                // Variable aufbereiten
                sizeData = sizeof( uint32_t );
                // Datengroesze 
                sizeData += ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? uint32_t( count - addressOffset ) : IV_FLASH_SIZE_DATA;

                // Zu setzende Daten uebernehmen
                for (uint16_t i = sizeof( uint32_t ); i < sizeData; ++i)
                {
                    m_packet_data[i] = buffer[addressOffset + (i - sizeof( uint32_t ))];
                }

                // Bei aktivem Blocktransfer die bereits uebertragene Datenmenge halten.
                if (controllerSegmentCurrent.transfer_state == controller_segment::bts_active)
                {
                    controllerSegmentCurrent.size += (sizeData - sizeof( uint32_t ));
                }

                if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
                {
                    // Fehler ausgeben
                    trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, result);
                    return result;
                }

                // Keine Antwortdaten erwartet
                if (sizeData)
                {
                    trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
                }

                if ( m_call_back_function_with_parameters != nullptr )
                {
                    if ( m_call_back_function_with_parameters( (uint32_t)count, uint32_t( addressOffset + sizeData ) ) == 0 )
                    {
                        result = -IV_ERROR_BREAK;
                        trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, result );
                        break;
                    }
                }


            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        if ( m_segments.size() && m_segments.back().file_stream->is_open() )
        {
            m_segments.back().file_stream->write( reinterpret_cast<char*>( buffer ), count );
            // Wartezeit simulieren
            std::this_thread::sleep_for( std::chrono::milliseconds( 50 ) );
        }

        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_from_flash(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_FLASH_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Angeforderte Laenge eintragen
            *(uint16_t *)&m_packet_data[sizeof( uint32_t )] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof( uint32_t ) + sizeof( uint16_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, result);
                return result;
            }

            if ( m_call_back_function_with_parameters != nullptr )
            {
                 if ( !m_call_back_function_with_parameters( (uint32_t)count, uint32_t( addressOffset + sizeData ) ) )
                 {
                    result = -IV_ERROR_BREAK;
                    trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, result );
                    break;
                 }
             }

            // OK

            // Empfangene Daten uebernehmen
            for (uint16_t i=0; i < (sizeData - sizeof( uint32_t )); i++)
            {
                buffer[addressOffset + i] = m_packet_data[sizeof( uint32_t ) + i];
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_from_flash_physical(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    uint32_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_FLASH_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Angeforderte Laenge eintragen
            *(uint16_t *)&m_packet_data[sizeof( uint32_t )] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof( uint32_t ) + sizeof( uint16_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL, result);
                return result;
            }

            if ( m_call_back_function_with_parameters != nullptr )
            {
                if ( !m_call_back_function_with_parameters( (uint32_t)count, uint32_t( addressOffset + sizeData ) ) )
                {
                    result = -IV_ERROR_BREAK;
                    trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL, result );
                    break;
                }
            }

            // OK

            // Empfangene Daten uebernehmen
            for (uint16_t i = 0; i < (sizeData - sizeof( uint32_t )); i++)
            {
                buffer[addressOffset + i] = m_packet_data[sizeof( uint32_t ) + i];
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

#define IV_EEPROM_SIZE_DATA   256

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_to_eeprom(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_EEPROM_SIZE_DATA;

            // Memberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Zu setzende Daten uebernehmen
            for ( uint16_t i=0; i < sizeData; i++ )
            {
                m_packet_data[sizeof( uint32_t ) + i] = buffer[addressOffset + i];
            }

            // Variable aufbereiten
            sizeData += sizeof( uint32_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, result);
                return result;
            }

            // Keine Antwortdaten erwartet
            if (sizeData)
            {
                trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
            }

            //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(uint32_t*)_packetData, *(uint16_t*)&_packetData[sizeof(uint32_t)]);
            /*if (_callBackFunction) {
            if (!_callBackFunction()) {

            // Fehlernummern setzen
            m_errorCommunicationNum = IV_ERROR_BREAK;
            // Fehlertext ermitteln und eintragen.
            GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

            Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
            }
            } else*/ if ( m_call_back_function_with_parameters != nullptr )
            {
                if ( !m_call_back_function_with_parameters( (uint32_t)count, uint32_t( addressOffset + sizeData ) ) )
                {
                    result = -IV_ERROR_BREAK;
                    trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, result );
                    break;
                }
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_from_eeprom(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_EEPROM_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Angeforderte Laenge eintragen
            *(uint16_t *)&m_packet_data[sizeof( uint32_t )] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof( uint32_t ) + sizeof( uint16_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM, result);
                return result;
            }

            // OK

            // Empfangene Daten uebernehmen
            for (uint16_t i=0; i < (sizeData - sizeof( uint32_t )); i++)
            {
                buffer[addressOffset + i] = m_packet_data[sizeof( uint32_t ) + i];
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}


#define IV_RAM_SIZE_DATA   256

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_to_ram(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_RAM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_RAM_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_RAM_SIZE_DATA;

            // Memberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Zu setzende Daten uebernehmen
            for (uint16_t i=0; i < sizeData; i++)
            {
                m_packet_data[sizeof( uint32_t ) + i] = buffer[addressOffset + i];
            }

            // Variable aufbereiten
            sizeData += sizeof( uint32_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM, result);
                return result;
            }

            // Keine Antwortdaten erwartet
            if (sizeData)
            {
                trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
            }

            //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(uint32_t*)_packetData, *(uint16_t*)&_packetData[sizeof(uint32_t)]);
            /*if (_callBackFunction) {
            if (!_callBackFunction()) {

            // Fehlernummern setzen
            m_errorCommunicationNum = IV_ERROR_BREAK;
            // Fehlertext ermitteln und eintragen.
            GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

            Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
            }
            } else*/ if ( m_call_back_function_with_parameters != nullptr )
            {
                if ( !m_call_back_function_with_parameters( (uint32_t)count, uint32_t( addressOffset + sizeData ) ) )
                {
                    result = -IV_ERROR_BREAK;
                    trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM, result );
                    break;
                }
            }
        }	
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_from_ram(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_RAM_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Angeforderte Laenge eintragen
            *(uint16_t *)&m_packet_data[sizeof( uint32_t )] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof( uint32_t ) + sizeof( uint16_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RAM, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RAM, result);
                return result;
            }

            // OK

            // Empfangene Daten uebernehmen
            for (uint16_t i=0; i < (sizeData - sizeof( uint32_t )); i++)
            {
                buffer[addressOffset + i] = m_packet_data[sizeof( uint32_t ) + i];
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RAM, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* 2014_10_09(BOt)
* Ab der Version 04.03.04 wird der Parameter count für die Anfrage als in/out verwendet,
* vorher wurden immer Konstant 532 Byte angefordert, der Parameter war lediglich out-Value.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_flash_page(uint8_t device_address, std::size_t page_number, std::size_t offset, uint8_t *buffer, std::size_t &count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, 0x%08X, %u ]\n", __FILE__, __LINE__, __func__, device_address, page_number, buffer, count);

    int result = IV_ERROR_SUCCESS;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Meberpuffer aufbereiten
        *(uint16_t *)m_packet_data = uint16_t( page_number );
        // Offset in die Seite
        *(uint16_t *)&m_packet_data[sizeof( uint16_t )] = uint16_t( offset );
        // Angeforderte Laenge eintragen
        *(uint16_t *)&m_packet_data[sizeof( uint16_t ) * 2] = uint16_t( count );

        // Für späteren Vergleich halten.
        uint16_t countIn = (uint16_t)count;
        // Variable aufbereiten
        count = sizeof( uint16_t ) * 3;

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE, m_packet_data, (uint16_t &)count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE, result);
            return result;
        }

        // OK
        if ( countIn != (count - (sizeof( uint16_t ) * 2)) )
        {
            trace("%s(%i):ERROR(1): %s [ Die geforderte Anzahl Bytes wrude nicht geliefert. (in: %i, out: %i) ]\n", __FILE__, __LINE__, __func__, countIn, (count - sizeof(uint16_t) - 2));
        }

        // Empfangene Daten uebernehmen
        for (uint16_t i=0; i < (count - (sizeof( uint16_t ) * 2)); i++)
        {
            buffer[i] = m_packet_data[sizeof( uint16_t ) * 2 + i];
        }
        count -= sizeof( uint16_t ) * 2;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// Groesse fuer Datenblock
#define IV_MMC_SIZE_DATA      512

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_to_mmc(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, %p, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_MMC_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_MMC_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_MMC_SIZE_DATA;

            // Memberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Zu setzende Daten uebernehmen
            for (uint16_t i=0; i < sizeData; i++)
            {
                m_packet_data[sizeof( uint32_t ) + i] = buffer[addressOffset + i];
            }

            // Variable aufbereiten
            sizeData += sizeof( uint32_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC, result);
                return result;
            }

            // Keine Antwortdaten erwartet
            if (sizeData)
            {
                trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
            }

            //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(uint32_t*)_packetData, *(uint16_t*)&_packetData[sizeof(uint32_t)]);
            /*if (_callBackFunction) {
            if (!_callBackFunction()) {

            _result = -IV_ERROR_BREAK;
            // Fehlernummern setzen
            m_errorCommunicationNum = IV_ERROR_BREAK;
            // Fehlertext ermitteln und eintragen.
            GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

            Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
            }
            } else*/ if ( m_call_back_function_with_parameters != nullptr )
            {
                if ( !m_call_back_function_with_parameters( (uint32_t)count, uint32_t( addressOffset + sizeData ) ) )
                {
                    result = -IV_ERROR_BREAK;
                    trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC, result );
                    break;
                }
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_from_mmc(uint8_t device_address, std::size_t address, uint8_t *buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %u, %p, %u ]\n", __FILE__, __LINE__, __func__, device_address, address, buffer, count);

    int result = IV_ERROR_SUCCESS;
    std::size_t addressOffset;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_MMC_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_MMC_SIZE_DATA) > count) ? uint16_t( count - addressOffset ) : IV_MMC_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(uint32_t *)m_packet_data = uint32_t( address + addressOffset );
            // Angeforderte Laenge eintragen
            *(uint16_t *)&m_packet_data[sizeof( uint32_t )] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof( uint32_t ) + sizeof( uint16_t );

            if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_MMC, m_packet_data, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_MMC, result);
                return result;
            }

            // OK

            // Empfangene Daten uebernehmen
            for (uint16_t i=0; i < (sizeData - sizeof( uint32_t )); i++)
            {
                buffer[addressOffset + i] = m_packet_data[sizeof( uint32_t ) + i];
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_MMC, tmp, (uint16_t &)count )) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die Struktur zum SFS.
*
* @param device_address Anzusprechende Geraetenummer.
* @param serial Variable die die ermittelte Seriennummer erhaehlt.
*/
int Datafox::Communication::Module_system::read_sfs_mpb(uint8_t device_address, uint8_t &type, SFSMPB &sfs)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SFSMPB, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SFSMPB, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count != (sizeof(uint8_t) + sizeof(SFSMPB)))
        {
            trace("%s(%i):WARNING(1): %s [ SFS ungleich der erwarteten Größe ]\n", __FILE__, __LINE__, __func__);
            type = 0;
            memset(&sfs, 0, sizeof(SFSMPB));
        }
        else
        {
            type = *m_packet_data;
            sfs = *(SFSMPB*)&m_packet_data[1];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[64];
        count = sizeof(uint8_t) + sizeof(SFSMPB);
        memset(tmp, 0, sizeof(tmp));

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SFSMPB, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die Firmwareversion.
*
* @param device_address Anzusprechende Geraetenummer.
* @param version Zeiger Variable fuer den Versionsstring.
*                Format: XX.XX.XX.XX ?...
*/
int Datafox::Communication::Module_system::read_firmware_version(uint8_t device_address, std::string &version)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, version.c_str());

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION, result);
            return result;
        }

        // OK
        version = reinterpret_cast<char *>(&m_packet_data[0]);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        std::vector<std::string> short_names;
        short_names.push_back( "PZE" ); // Wert für 0
        short_names.push_back( "ZK" );
        short_names.push_back( "BDE" );
        short_names.push_back( "MOBIL" );
        short_names.push_back( "MDE" );
        short_names.push_back( "AE" );
        short_names.push_back( "TIMEBOY" );
        short_names.push_back( "EXKLUSIV" );
        short_names.push_back( "FLEX" );
        short_names.push_back( "TB-MOBIL-PZE" );
        short_names.push_back( "Evo28" );
        short_names.push_back( "Evo43" );
        short_names.push_back( "VARIO" );
        short_names.push_back( "IPC" );
        short_names.push_back( "Q7" );
        short_names.push_back( "IOBOX" );
        short_names.push_back( "ZKBOX" );
        short_names.push_back( "MOBILBOX" );
        short_names.push_back( "DockingV2" );
        short_names.push_back( "LoggerV2" );
        short_names.push_back( "ZKKNOTEN" );
        short_names.push_back( "EVO35U" );
        short_names.push_back( "SAMS-OCL" );
        short_names.push_back( "EVO35" );
        short_names.push_back( "ZKREADERPLUS" );
        short_names.push_back( "ZKZENTRALE" );


        uint8_t tmp[64];
        std::string short_name("?");
        if ( short_names.size() > get_property( "hardware_info.type_device", 0xFF ) )
        {
            short_name = short_names[get_property( "hardware_info.type_device", 0 )];
        }
        count = sprintf((char*)tmp, "04.03.10.02.%s", short_name.c_str()) + 1;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        return 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die Bootloaderversion.
*
* @param device_address Anzusprechende Geraetenummer.
* @param version Zeiger auf Variable fuer Versionsstring. 
*                Format: XX.XX.XX.XX ?...
*/
int Datafox::Communication::Module_system::read_bootloader_version(uint8_t device_address, std::string &version)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, version.c_str());

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER, result);
            return result;
        }

        // OK
        version = reinterpret_cast<char *>(&m_packet_data[0]);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[64];
        count = sprintf((char*)tmp, "04.01.04.99") + 1;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die Seriennummer des Geraets.
*
* @param device_address Anzusprechende Geraetenummer.
* @param serial Variable die die ermittelte Seriennummer erhaehlt.
*/
int Datafox::Communication::Module_system::read_serial_number(uint8_t device_address, uint32_t &serial)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, serial);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count > (int)sizeof(uint32_t))
        {
            trace("%s(%i):WARNING(1): %s [ Seriennummer mit %i Byte, laenger als erwartete 32bit ]\n", __FILE__, __LINE__, __func__, count - 4);
        }

        serial = *(uint32_t*)m_packet_data;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[64];
        *(uint32_t*)tmp = get_property( "device.serial", -1 );
        count = sizeof(uint32_t);

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        return 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Setzten der RTC (RealTimeClock) im Geraet.
*
* @param device_address Anzusprechende Geraetenummer.
* @param time Zeitstring im Format: "JJJJ-MM-DD hh:mm:ss"
*/
int Datafox::Communication::Module_system::write_date_time(uint8_t device_address, char *dt)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, dt);

    int result = IV_ERROR_SUCCESS;
    int i;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        count = 20;
        // Puffer loeschen
        memset(m_packet_data, 0, sizeof(m_packet_data));
        for (i=0; i < (count - 1); i++)
        {
            m_packet_data[i] = dt[i];
        }

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Lesen der RTC (RealTimeClock) aus Geraet.
*
* @param device_address Anzusprechende Geraetenummer.
* @param time Zeiger auf Zeichenkette fuer Zeitstring. 
*             Format: "JJJJ-MM-DD hh:mm:ss"
*/
int Datafox::Communication::Module_system::read_date_time(uint8_t device_address, char *dt)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, dt);

    int result = IV_ERROR_SUCCESS;
    int i;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count > 20)
        {
            trace("%s(%i):WARNING(1): %s [ Uhrzeitstring ist mit %i Byte laenger als erwartet. ]\n", __FILE__, __LINE__, __func__, count - 20);
        }

        // OK
        for (i=0; i < count; i++)
        {
            dt[i] = m_packet_data[i];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        std::string dt( 30, '\0' );
        std::time_t t = std::time( nullptr );
        struct tm *l = std::localtime( &t );
        std::strftime( &dt[0], dt.capacity() - 1, "%Y-%m-%d %H:%M:%S", l );
        uint16_t count = dt.find( '\0' ) + 1;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC, (uint8_t *)&dt[0], count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

uint8_t Datafox::Communication::Module_system::hex_to_uint8(char *data)
{
    int i;
    uint8_t val = 0;

    for (i=0; i < 2; i++)
    {
        val = (val << 4);

        if (data[i] >= '0' && data[i] <= '9')
        {
            val += data[i] - '0';
        }
        else if (data[i] >= 'a' && data[i] <='f')
        {
            val += data[i] - 'a' + 10;
        }
        else
        {
            val += data[i] - 'A' + 10;
        }
    }

    return val;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_system_variable( uint8_t device_address, const std::string &name, const std::string &val )
{
    trace( "%s(%i):INFO(3): %s [ %i, %s, %s ]\n", __FILE__, __LINE__, __func__, device_address, name.c_str(), val.c_str() );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;
    std::string lName;
    std::string lValue;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Name für Vergleich übernehmen
        lName = name;
        for_each(lName.begin(), lName.end(), [](char &c){ c = std::toupper(c, std::locale()); });

        // Wert für ggf. Anpassungen übernehmen
        lValue = val;

        if ( (lName.find("MOBILE.") == 0 || lName.find("_MODEM_MC35I.") == 0) 
            && (lName.find(".GPRS") != std::string::npos || lName.find(".USER") != std::string::npos || lName.find(".PASSWORD") != std::string::npos))
        {
            std::string version;            
            if ( (result = read_firmware_version( device_address, version )) != IV_ERROR_SUCCESS )
                return result;

            SoftwareVersion ver(version);
            if (ver >= "04.01.06.00")
                latin1_to_gsm_codec(lValue);
        }

        // Puffer loeschen
        memset(m_packet_data, 0, sizeof(m_packet_data));

        // Namen uebernehmen
        std::copy(name.cbegin(), name.cend(), &m_packet_data[count]);
        count += static_cast<int>( name.size() );
        m_packet_data[count] = 0; // Nullterminierung
        ++count;

        // Wert uebernehmen
        std::copy(lValue.cbegin(), lValue.cend(), &m_packet_data[count]);
        count += static_cast<int>( lValue.size() );
        m_packet_data[count] = 0;
        ++count;

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;
        std::string n = boost::to_lower_copy( name );
        if ( !m_variables.count( n ) )
        {
            m_variables.insert( std::make_pair( n, val ) );
        }
        else
        {
            m_variables[n] = val;
            /*
            uint16_t e = 0x0001; // Majorbestandteil unbekannt.
            uint16_t count = sizeof( uint16_t );
            // Antwort des Befehls senden.
            if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, (uint8_t *)&e, count)) != IV_ERROR_SUCCESS)
            {
                trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
                return _result;
            }
            */
        }

        for ( auto &v : m_variables )
        {
            trace( "Variable: %s = %s\n", v.first.c_str(), v.second.c_str() );
        }

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        return 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_system_variable(uint8_t device_address, const std::string &name, std::string &val)
{
    trace("%s(%i):INFO(3): %s [ %i, %s, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, name.c_str(), val.c_str());

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;
    std::string lName;
    std::string lValue; // Größe aufgrund von SYSTEM.STATUS nötig.

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Puffer loeschen
        memset(m_packet_data, 0, sizeof(m_packet_data));

        // Namen uebernehmen
        std::copy(name.cbegin(), name.cend(), &m_packet_data[count]);
        count += static_cast<int>( name.size() );
        m_packet_data[count] = 0; // Nullterminierung
        ++count;

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, m_packet_data, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count > 80)
        {
            trace("%s(%i):WARNING(3): %s [ Laenge der Variablen mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, __func__, count);
        }

        count = 0;
        // Namen uebergehen
        lName = reinterpret_cast<char *>(&m_packet_data[count]);	
        for_each(lName.begin(), lName.end(), [](char &c){ c = std::toupper(c, std::locale()); });
        count = static_cast<int>( lName.size() ) + 1;

        // Wert uebernehmen
        lValue = reinterpret_cast<char *>(&m_packet_data[count]);	

        // Name für Vergleich übernehmen
        if ( (lName.find("MOBILE.") == 0 || lName.find("_MODEM_MC35I.") == 0) 
            && (lName.find(".GPRS") != std::string::npos || lName.find(".USER") != std::string::npos || lName.find(".PASSWORD") != std::string::npos))
        {
            std::string version;            
            if ( (result = read_firmware_version( device_address, version )) != IV_ERROR_SUCCESS )
                return result;

            SoftwareVersion ver(version);
            if (ver >= "04.01.06.00")
                gsm_to_latin1_codec(lValue);
        }

        // Wert uebernehmen
        val = lValue;	
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        std::string n = boost::to_lower_copy( name );
        if ( m_variables.count( n ) )
        {
            std::vector<uint8_t> buffer;
            std::string value( m_variables[n] );
            buffer.resize( n.size() + 1 + value.size() + 1, 0 );
            n.copy( reinterpret_cast<char *>( &buffer[0] ), n.size() );
            value.copy( reinterpret_cast<char *>( &buffer[n.size() + 1] ), value.size() );

            uint16_t count = buffer.size();
            // Antwort des Befehls senden.
            if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, &buffer[0], count)) != IV_ERROR_SUCCESS)
            {
                trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
                return result;
            }
        }
        else if ( n == "system.status" || n == "system.status2" )
        {
            std::vector<uint8_t> buffer;
            std::string value( "virtual device" );
            buffer.resize( n.size() + 1 + value.size() + 1, 0 );
            n.copy( reinterpret_cast<char *>( &buffer[0] ), n.size() );
            value.copy( reinterpret_cast<char *>( &buffer[n.size() + 1] ), value.size() );

            uint16_t count = buffer.size();
            // Antwort des Befehls senden.
            if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, &buffer[0], count)) != IV_ERROR_SUCCESS)
            {
                trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
                return result;
            }
        }
        else
        {
            uint16_t e = 0x0001; // Majorbestandteil unbekannt.
            uint16_t count = sizeof( uint16_t );
            // Antwort des Befehls senden.
            if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, (uint8_t *)&e, count)) != IV_ERROR_SUCCESS)
            {
                trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
                return result;
            }
        }
#else
        return 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_relay_state(uint8_t device_address, const std::string &name, uint8_t bus_number, uint8_t relay_number, uint8_t status, uint16_t time)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, name.c_str(), bus_number, relay_number, status, time);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Parameter uebernehmen
        std::copy(name.cbegin(), name.cend(), &m_packet_data[count]);
        count += static_cast<int>( name.size() );
        m_packet_data[count] = 0; // Nullterminierung
        ++count;

        // Weitere Parameter als String uebergehen
        count += sprintf((char*)&m_packet_data[count], "%i", bus_number) + 1;
        count += sprintf((char*)&m_packet_data[count], "%i", relay_number) + 1;
        count += sprintf((char*)&m_packet_data[count], "%i", status) + 1;
        count += sprintf((char*)&m_packet_data[count], "%i", time) + 1;
        /*
        // Busnummer
        _packetData[count] = busNum;
        count += sizeof(uint8_t);

        // Relaisnummer
        _packetData[count] = relayNum;
        count += sizeof(uint8_t);

        // Status
        _packetData[count] = status;
        count += sizeof(uint8_t);

        // Time
        *(uint16_t*)&_packetData[count] = status;
        count += sizeof(uint16_t);
        */

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_relay_state(uint8_t device_address, const std::string &name, uint8_t bus_number, uint8_t relay_number, uint8_t &status, uint16_t &time)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, name.c_str(), bus_number, relay_number, status, time);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);


        // Namen uebernehmen
        std::copy(name.cbegin(), name.cend(), &m_packet_data[count]);
        count += static_cast<int>( name.size() );
        m_packet_data[count] = 0; // Nullterminierung
        ++count;

        // Weitere Parameter als String uebernehmen
        count += sprintf((char*)&m_packet_data[count], "%i", bus_number) + 1;
        count += sprintf((char*)&m_packet_data[count], "%i", relay_number) + 1;
        /*
        // Busnummer
        _packetData[count] = busNum;
        count += sizeof(uint8_t);

        // Relaisnummer
        _packetData[count] = relayNum;
        count += sizeof(uint8_t);
        */	
        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY, result);
            return result;
        }

        // Name
        count = (int)strlen((char*)m_packet_data) + 1;

        // Weitere Parameter als String uebergehen
        // busNum, relayNum
        count += (int)strlen((char*)&m_packet_data[count]) + 1;
        count += (int)strlen((char*)&m_packet_data[count]) + 1;

        // Status uebernehmen
        status = atoi((char*)&m_packet_data[count]);
        count += (int)strlen((char*)&m_packet_data[count]) + 1;

        // Zeit uebernehmen
        time = atoi((char*)&m_packet_data[count]);
        /*
        // Keine Antwortdaten erwartet
        if (count != (sizeof(uint8_t) + sizeof(uint16_t))) {

        Trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Status
        status = _packetData[0];
        // Time
        time = *(uint16_t*)&_packetData[sizeof(uint8_t)];
        */	
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_bios_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS, m_packet_data, count, 1)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
        /*	
        // Nach dem Wechsel ins Bios wird eine Bestaetigung gesendet.
        short module, func, error = IV_ERROR_SUCCESS;
        if ((_result = Pending(device_address, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
        Trace("%s(%i): %s() Quittierungsantwort - schlug fehl.\n", __FILE__, __LINE__, __func__);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_ENTERED_BIOS))
        {			
        Trace("%s(%i): %s() Antwort OK.\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS);
        return _result;
        }
        */
        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS, result);
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_enter_bios_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::exit_bios_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Beendigung einer Uebertragung eines Datanblocks
        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        /*	
        // Nach dem Verlassen des Bios kommt eine Bestaetigungsanfrage.
        short module, func, error = IV_ERROR_SUCCESS;
        if ((_result = Pending(device_address, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
        Trace("%s(%i):ERROR(1): %s [ Quittierungsantwort - schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_EXITED_BIOS))
        {			
        Trace("%s(%i):INFO(3): %s [ Antwort OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS);
        return _result;
        }
        */
        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS, result);
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_exit_bios_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_main_interface_type(uint8_t device_address, uint8_t type)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, type);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Den Geraetetyp uebernehmen.
        m_packet_data[0] = type;
        count += sizeof(uint8_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_main_interface_type(uint8_t device_address, uint8_t &type)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, type);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE, result);
            return result;
        }

        // OK
        if (count != sizeof(uint8_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        type = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_baud(uint8_t device_address, uint32_t baud)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, baud);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Den Geraetetyp uebernehmen.
        *(uint32_t*)&m_packet_data[0] = baud;
        count += sizeof(uint32_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_baud(uint8_t device_address, uint32_t &baud)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, baud);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD, result);
            return result;
        }

        // OK
        if (count != sizeof(uint32_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        baud = *(uint32_t*)m_packet_data;
        switch (baud)
        { // Bereich pruefen
        case 9600:
        case 14400:
        case 19200:
        case 38400:
        case 57600:
        case 115200:
            break;

        default:
            baud = 38400;
        }	

    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_device_address(uint8_t device_address, uint16_t address)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Den Geraetetyp uebernehmen.
        *(uint16_t*)&m_packet_data[0] = address;
        count += sizeof(uint16_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_device_address(uint8_t device_address, uint16_t &address)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID, result);
            return result;
        }

        // OK
        if (count != sizeof(uint16_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        address = *(uint16_t*)m_packet_data;
        if ( address < 1 || address > 254 )
            address = 254;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_bootloader_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER, m_packet_data, count, 1)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_xport_monitor_mode(uint8_t device_address, char* data, uint16_t &count)
{
    trace("%s(%i):INFO(3): %s [ %u, %p ]\n", __FILE__, __LINE__, __func__, device_address, data);

    int result = IV_ERROR_SUCCESS;
    uint16_t lCount = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR, m_packet_data, lCount)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (lCount)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK, result);
            return result;
        }

        // Daten uebernehmen
        for (uint16_t i=0; i < count; i++)
        {
            data[i] = m_packet_data[i];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_xport_monitor_mode_ext(uint8_t device_address, char* data, uint16_t &count)
{
    trace("%s(%i):INFO(3): %s [ %u, %p ]\n", __FILE__, __LINE__, __func__, device_address, data);

    int result = IV_ERROR_SUCCESS;
    uint16_t lCount = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT, m_packet_data, lCount)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (lCount)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK, result);
            return result;
        }

        // Daten uebernehmen
        for (uint16_t i=0; i < count; i++)
        {
            data[i] = m_packet_data[i];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_enter_xport_monitor_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_enter_xport_monitor_mode_ext(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::xport_monitor_mode_command(uint8_t device_address, char* buffer, uint16_t &count)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, device_address, buffer, count);

    int result = IV_ERROR_SUCCESS;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Daten fuer senden uebernehmen
        for (uint16_t i=0; i < count; i++)
        {
            m_packet_data[i] = buffer[i];
        }

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT, result);
            return result;
        }

        // OK

        // Daten von XPort uebernehmen
        for (uint16_t i=0; i < count; i++)
        {
            buffer[i] = m_packet_data[i];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::exit_xport_monitor_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Beendigung einer Uebertragung eines Datanblocks
        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }


        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK, result);
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_exit_xport_monitor_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_com_server_mode(uint8_t device_address, const char* comm, uint32_t baud, uint8_t byte_size, uint8_t parity, uint8_t stop_bits, uint16_t flags, uint16_t timeout_to_close)
{
    trace("%s(%i):INFO(3): %s [ %i, %s, %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, comm, baud, byte_size, parity, stop_bits, timeout_to_close);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Alle Zusatzparameter uebernehmen
        count += sprintf((char*)&m_packet_data[count], "%s", comm) + 1;
        *(uint32_t*)&m_packet_data[count] = baud;
        count += sizeof(uint32_t);
        m_packet_data[count] = byte_size;
        count += sizeof(uint8_t);
        m_packet_data[count] = parity;
        count += sizeof(uint8_t);
        m_packet_data[count] = stop_bits;
        count += sizeof(uint8_t);
        *(uint16_t*)&m_packet_data[count] = flags;
        count += sizeof(uint16_t);
        *(uint16_t*)&m_packet_data[count] = timeout_to_close;
        count += sizeof(uint16_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Aktive Antwort abwarten
        if ((result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK, result);
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_enter_com_server_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}
// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::exit_com_server_mode(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die Information zu einem Modultyp
*
* @param device_address Anzusprechende Geraetenummer.
* @param name Das anzusprechende Modul. (ACCESS.MODULE fuer Zutrittskontrollmodul)
* @param param Abhaengig vom verwendeten Namen. (Bei ACCESS.MODULE gibt es die TM-Nummer des Moduls )
* @param info Zeiger auf Zeichenkettenpuffer fuer Rueckgabe.
* @param infoLength Laenge des Zeichenkettenpuffers fuer Rueckgabe
*/
int Datafox::Communication::Module_system::read_hardware_module_info(uint8_t device_address, const std::string &name, const std::string &param, std::string &info)
{
    trace("%s(%i):INFO(3): %s [ %i, %s, %s, %s, %i ]\n", __FILE__, __LINE__, __func__, device_address, name.c_str(), param.c_str(), info.c_str());

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        std::string param_( param );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);
 
        // Ab der Version 04.03.04 kann bei ACCESS.MODULE bei param noch mit CR die Bus-Id angefügt werden,
        // Vorgängerversionen verstehen diese Angabe nicht und sie muss daher ersetzt werden.
        if ( name == "ACCESS.MODULE" )
        {
            int res;
            std::string regTM("[0-3]\\d[01](\\r\\d{1,4})?");

            // Format pruefen

            if ( !std::regex_match( param_, std::regex( regTM ) ) )
                return -IV_ERROR_FUNCTION_PARAMETER;

            if ( param_.find_first_of( '\r' ) != std::string::npos )
            {
                std::string v;

                // Firmwareversion ermitteln.
                if ( (res = read_firmware_version( device_address, v )) != IV_ERROR_SUCCESS )
                    return res;

                SoftwareVersion version(v);
                // Kleiner dieser Version muss zusätzlich geprüft werden, dass die angegebene MasterId mit der im 
                // Geraet eingestellten uebereinstimmt. Sie wird nicht als Parameter mitgesendet.
                if ( version < "04.03.04" )
                {
                    // Aufteilen des Strings in seine Bestandteile.
                    std::vector<std::string> tokens;
                    boost::split( tokens, param_, boost::is_any_of( "\r" ) );
                    if ( tokens.size() < 2 )
                        return -IV_ERROR_FUNCTION_PARAMETER;

                    // Angegebene MasterId gegen die eingestellte prüfen.
                    std::string masterId;

                    // Systemvariable ACCESS.MASTERID ermitteln.
                    if ( (res = read_system_variable( device_address, "ACCESS.MASTERID", masterId )) != IV_ERROR_SUCCESS )
                        return res;

                    // Prüfen ob die richtige MasterId angesprochen werden soll.
                    if ( std::stoi( masterId ) != std::stoi( tokens[1] ) )
                        return -IV_ERROR_FUNCTION_PARAMETER;

                    // Terminierung hinter der Türmodulangabe, da diese Firmwareversion die angefuegte MasterId nicht unterstuetzt.
                    param_[tokens[0].size()] = '\0';
                }
            }
        }

        count += static_cast<int>( name.copy(reinterpret_cast<char *>(&m_packet_data[count]), std::string::npos) );
        m_packet_data[count] = 0; // Nullterminierung
        ++count;

        count += static_cast<int>( param_.copy(reinterpret_cast<char *>(&m_packet_data[count]), std::string::npos) );
        m_packet_data[count] = 0;
        ++count;

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO, result);
            return result;
        }

        // Antwortdaten erwartet
        if ( (unsigned)count < (name.size() + 1 + param_.size() + 1) )
        {
            trace("%s(%i):WARNING(1): %s [ Antwort ist zu kurz. ]\n", __FILE__, __LINE__, __func__);
        }

        std::string name_(reinterpret_cast<char *>(&m_packet_data[0]));
        // Namen oder leerer Namen zulaessig.
        if (name_.size() == name.size() || name_.size() == 0)
        {
            param_ = reinterpret_cast<char *>(&m_packet_data[name_.size() + 1]);
            if (param_.size() <= param.size())
            {
                // Wert uebernehmen.
                info = reinterpret_cast<char *>(&m_packet_data[name_.size() + 1 + param_.size() + 1]);
                std::string test( info );
                for_each(test.begin(), test.end(), [](char &c){ c = std::tolower(c, std::locale()); });
                if ( test == "active-answer" )
                {
                    if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO_OK, m_packet_data, count, IV_PENDING_TRIALS(get_connection().get_read_timeout()) )) != IV_ERROR_SUCCESS )
                    {
                        trace("%s(%i):ERROR(1): %s() [ Informationsabfrage schlug fehl. ]\n", __FILE__, __LINE__, __func__);
                        return result;
                    }

                    info =  reinterpret_cast<char *>( &m_packet_data[0] );
                }
            }
            else
            {
                result = -IV_ERROR_INTERN;
            }
        }
        else
        {
            result = -IV_ERROR_INTERN;
        }        
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;


        std::string name_( name );
        std::string param_( param );

        if ( name_ == "COM.TCPIP" )
        {
            std::array<uint8_t, sizeof( struct sockaddr )> sa_buf;
            struct sockaddr *sa = (struct sockaddr *)sa_buf.data();
            struct sockaddr_in *sa_in = (struct sockaddr_in *)sa;
            std::fill( sa_buf.begin(), sa_buf.end(), 0 );
            socklen_t sa_size = sa_buf.size();
            getsockname( get_connection().get_socket(), sa, &sa_size );
            std::string v = boost::str( boost::format( "%s:%d" ) % inet_ntoa( sa_in->sin_addr ) % sa_in->sin_port );
            trace( "Abfrage auf IP: %s", v.c_str() );
        }

        return 0x0001;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::reply_read_module_info(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        count = sizeof(uint16_t);
        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO_OK, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(get_device_address_receiver(), IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO_OK, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Aktivieren oder Deaktivieren einer Benachrichtigung.
*
* @param device_address Anzusprechende Geraetenummer.
* @param type Benachrichtigungstyp
* @param state Zu setzender Zustand.
*/
int Datafox::Communication::Module_system::write_notify(uint8_t device_address, uint8_t type, uint8_t state)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, type, state);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        m_packet_data[count] = type;
        count += sizeof(uint8_t);
        m_packet_data[count] = state;
        count += sizeof(uint8_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}


// ##################################################################
/**
* @brief Aktivieren oder Deaktivieren einer Benachrichtigung.
*
* @param device_address Anzusprechende Geraetenummer.
* @param type Benachrichtigungstyp
* @param state Aktuell gesetzter Zustand.
*/
int Datafox::Communication::Module_system::read_notify(uint8_t device_address, uint8_t type, uint8_t &state)

{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, type, state);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        m_packet_data[count] = type;
        count += sizeof(uint8_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY, result);
            return result;
        }

        // Antwortdaten erwartet
        if (count != sizeof(uint8_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwort ist zu lang oder kurz. ]\n", __FILE__, __LINE__, __func__);
        }
        else
        {
            // Gemeldeten Zustand uebernehmen
            state = m_packet_data[0];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Abfragen der Softwareinfo der eingespielten Anwendungsfirmware.
*
* @param device_address Anzusprechende Geraetenummer.
* @param swi Puffer für die Softwareinfo.
* @param infoLength Länge des Puffers und übergebene Anzahl daten.
*/
int Datafox::Communication::Module_system::read_software_info(uint8_t device_address, uint8_t* info, int &info_length)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, device_address, info, info_length);

    int result = IV_ERROR_SUCCESS;
    int i;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO, result);
            return result;
        }

        // Antwortdaten erwartet
        for (i=0; i < count && i < info_length; i++)
        {
            info[i] = m_packet_data[i];
        }
        info_length = i;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_system_flags(uint8_t device_address, uint32_t flags)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, flags);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);


        // Den Geraetetyp uebernehmen.
        *(uint32_t*)&m_packet_data[0] = flags;
        count += sizeof(uint32_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_FLAGS, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_FLAGS, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_FLAGS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_system_flags(uint8_t device_address, uint32_t &flags)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, flags);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_FLAGS, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_FLAGS, result);
            return result;
        }

        // OK
        if (count != sizeof(uint32_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        flags = *(uint32_t*)m_packet_data;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_FLAGS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_system_record( uint8_t device_address, uint8_t *buffer, std::size_t &count, uint8_t &status)
{
    trace("%s(%i):INFO(3): %s [ %u, 0x08X, %u, %u ]\n", __FILE__, __LINE__, __func__, device_address, buffer, count, status);

    int result = IV_ERROR_SUCCESS;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYS_RECORD, m_packet_data, (uint16_t &)count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYS_RECORD, result);
            return result;
        }

        // OK
        switch (m_packet_data[0])
        {
        case IV_ANSW_NO_SYS_RECORD:         // kein Datensatz vorhanden
            status = 0;
            break;

        case IV_ANSW_ONE_SYS_RECORD:        // ein Systemdatensatz 
            status = 1;
            break;

        default:
            status = 0;
            break;
        }

        if (((status == 0) && (count == 0)) || (count > IV_SYS_RECORDS_MAX))
        {
            return -IV_ERROR_PROCESSING_DLL;
        }

        for (uint16_t i = 0; i < count; i++)
        {
            buffer[i] = m_packet_data[i];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t data[IV_SYS_RECORDS_MAX];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYS_RECORD, data, (uint16_t &)count )) != IV_ERROR_SUCCESS) {

            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int Datafox::Communication::Module_system::quit_system_record( uint8_t device_address, uint8_t &status)
{
    trace("%s(%i):INFO(3): %s [ %u, %u ]\n", __FILE__, __LINE__, __func__, device_address, status);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYS_RECORD, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYS_RECORD, result);
            return result;
        }

        // OK
        if (count > (int)sizeof(uint8_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        switch (m_packet_data[0])
        {
        case 0:
            status = 0;
            break;

        case IV_ANSW_ACK_ON_SYS_RECORD:        // Quittung erhalten (fuer Online-Datensatz) 
            status = 1;
            break;
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t data[1];

        ////if (m_records.size()) {

        ////	CRecord &record = m_records.front();

        ////	switch(record._packetData[0]) {
        ////	case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
        ////		data[0] = 0;
        ////		break;
        ////	case MB_ANSW_ONLINE_RECORD:         // Online Datensatz
        ////	case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz
        ////		data[0] = MB_ANSW_ACK_ON_RECORD;
        ////		break;
        ////	case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz
        ////	case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
        ////		data[0] = MB_ANSW_ACK_OFF_RECORD;
        ////		break;
        ////	}

        ////	count = sizeof(uint8_t);

        ////	// Datensatz loeschen
        ////	m_records.pop();
        ////} else {

        ////	data[0] = 0;
        ////	count = sizeof(uint8_t);
        ////}

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYS_RECORD, data, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;

}

int Datafox::Communication::Module_system::restore_system_records(uint8_t device_address)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_RESTORE_ALLSYS_RECORDS, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_RESTORE_ALLSYS_RECORDS, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        //		while (m_records.size()) {
        //			m_records.pop();
        //		}

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_RESTORE_ALLSYS_RECORDS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;

}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_setup_password(uint8_t device_address, const std::string &password)
{
    trace("%s(%i):INFO(3): %s [ %i, %s ]\n", __FILE__, __LINE__, __func__, device_address, password.c_str());

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Ob der Befehl unterstützt wird und das Gerät geschützt ist, intern halten.
        set_flag_password_protection(FALSE);
        set_flag_password_protected(FALSE);

        // Puffer loeschen
        memset(m_packet_data, 0, sizeof(m_packet_data));
        // Der Passwortpuffer wird mit 17 Zeichen verschluesselt
        count = 17;

        // Initialisieren der Verschluesselung
        get_protocol().df4_load_crypt_key();
        get_protocol().df4_init(TRUE);

        // Nur wenn Passwort angegeben wurde.
        if (password.size())
        {
            // Passwort uebernehmen
            password.copy(reinterpret_cast<char *>(&m_packet_data[0]), 16);
        }

        // Verschluesseln
        get_protocol().df4(m_packet_data, count);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK, result);

            // Fehler auswerten
            if (((result >> 8) & 0xFF) == 2)
            {
                // Funktion unbekannt
            }
            else if ((result & 0xFF) == 1)
            {
                // Funktionsfehler, passwort falsch
                // Passwortschutz ist möglich.
                set_flag_password_protection(TRUE);
            }

            return result;
        }

        // Befehl erfolgreich, also wird Passwortschutz voll genutzt.
        set_flag_password_protection(TRUE);
        set_flag_password_protected(TRUE);

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_setup_password_key(uint8_t device_address, char* key)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, key);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Keine Daten zu senden
        count = 0;

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_KEY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_KEY, result);
            return result;
        }

        // n Antwortdaten erwartet
        if (count != 17)
        {
            trace("%s(%i):WARNING(1): %s [ Laenge des Passwortschlüssels mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Wert uebernehmen
        strcpy(key, (char*)m_packet_data);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_KEY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_virtual_key(uint8_t device_address, uint8_t key_code, uint8_t flags)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, key_code, flags);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Keycode
        m_packet_data[count] = key_code;
        count += sizeof(uint8_t);

        // Flags
        m_packet_data[count] = flags;
        count += sizeof(uint8_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_PRESS_VIRTUAL_KEY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_PRESS_VIRTUAL_KEY, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_PRESS_VIRTUAL_KEY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_stream(uint8_t device_address, Stream_type type, const char* buffer, std::size_t count)
{
    trace("%s(%i):INFO(3): %s [ %u, %i, %p, %u ]\n", __FILE__, __LINE__, __func__, device_address, type, buffer, count);

    int result = IV_ERROR_SUCCESS;
    uint16_t sizeData;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Befehlsparamter aufbereiten.
        sizeData = 0;
        // Type der Streamdaten
        *(uint16_t *)m_packet_data = (uint16_t)type;
        sizeData += sizeof( uint16_t );
        // Offset falls der Stream wegen CRC-Fehler ab einer bestimmten Position neu gesendet werden soll. (Vorgesehen)
        *(uint32_t *)&m_packet_data[sizeData] = 0;
        sizeData += sizeof( uint32_t );
        // Anzahl Datenbyte des Streams
        if ( buffer )
        {
            *(uint32_t *)&m_packet_data[sizeData] = (uint32_t)count;
        }
        else
        {
            *(uint32_t *)&m_packet_data[sizeData] = 0;
        }
        sizeData += sizeof( uint32_t );
        // Stream-CRC
        if ( buffer )
        {
            *(uint32_t *)&m_packet_data[sizeData] = get_crc32( (uint8_t *)buffer, count );
        }
        else
        {
            *(uint32_t *)&m_packet_data[sizeData] = 0;
        }
        sizeData += sizeof( uint32_t );
        // Vorgesehenes Flagbyte fuer unterstuetzung des Blockweisen Streamings, und ggf. Erweiterungen.
        m_packet_data[sizeData] = 0;
        sizeData += sizeof( uint8_t );

        // Grundbefehl vor Sendung des Streams ausfuehren.
        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM, m_packet_data, sizeData )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM, result );
            return result;
        }

        // Liegt ein Stream vor?
        if ( !buffer )
        {
            return result;
        }

        char *bufBase = (char *)buffer;
        char *pos = bufBase;
        std::size_t bufCount = count;

        // Daten ggf. verschluesseln.
        std::string outCrypt;
        if ( get_protocol().is_crypt() )
        {
            outCrypt.assign( buffer, count );
            get_protocol().write_stream_crypt( outCrypt );

            bufBase = (char *)outCrypt.data();
            pos = bufBase;
            bufCount = outCrypt.size();
        }

        // Zu versendendes Stream-Protokoll-Paket erstellen.
        uint32_t uiTmp = uint32_t( bufCount + sizeof( uint16_t ) );
        uint16_t usTmp = 0;
        std::string prot;
        // Brutto Laenge ( 32bit ) eintragen.
        prot += (char)uiTmp & 0xFF;
        prot += (char)(uiTmp >> 8) & 0xFF;
        prot += (char)(uiTmp >> 16) & 0xFF;
        prot += (char)(uiTmp >> 24) & 0xFF;
        // Reserve ( 16bit ) eintragen.
        prot += (char)usTmp & 0xFF;
        prot += (char)(usTmp >> 8) & 0xFF;
        // Kopf-CRC berechnen und eintragen.
        usTmp = GetCRC16( (uint8_t *)prot.data(), (uint32_t)prot.size() );
        prot += (char)usTmp & 0xFF;
        prot += (char)(usTmp >> 8) & 0xFF;
        // Daten uebernehmen
        prot.append( pos, bufCount );
        // Daten-CRC berechnen und eintragen.
        usTmp = GetCRC16( (uint8_t *)pos, (uint32_t)bufCount );
        prot += (char)usTmp & 0xFF;
        prot += (char)(usTmp >> 8) & 0xFF;

        // Zeiger und Laengenangabe auf das Stream-Protokoll-Paket setzen.
        bufBase = (char *)prot.data();
        pos = bufBase;
        bufCount = prot.size();

        // Beim Kopieren des Stream-Protokoll-Paket werden alle 0x7E und 0x7F Bytes maskiert.
        std::string out;
        for ( ; (pos - bufBase) < bufCount; pos++ )
        {
            if ( *pos == 0x7E )
            {
                out += (char)0x7F;
                out += (char)0x01;                    
            }
            else if ( *pos == 0x7F )
            {
                out += (char)0x7F;
                out += (char)0x00;
            }
            else
            {
                out += *pos;
            }
        }

        // Befehlsausfuehrung geschuetzt durch Mutex.
        {
            lock_guard_call guard( m_mutex_call );

        }

        std::size_t transmittedCount = 0;
        // Stream-Protokoll-Paket senden.
        if ( (result = write_data( (const uint8_t *)&out[0], out.size(), transmittedCount )) != IV_ERROR_SUCCESS ) 
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM, result );
            return result;
        }

        if ( sizeData >= 2 )
        {
            int res = IV_ERROR_SUCCESS;
            std::size_t receiveCount;
            std::size_t receivedCount;
            bool masked;

            uint32_t crc = 0xFFFFFFFF;
            char *crcPos = (char *)buffer;
            uint32_t crcChunk = 0;
            int statusCount = 1;

            // In der Antwort werden folgende Werte geliefert:
            // - 8bit Flagbyte (reserviert)
            // - 8bit Die zu erwartende Groesse fuer die Status-Meldungen, in Byte.
            // - 32bit Maximale Puffergroesse, fuer Blockweises Streaming.

            uint8_t statusSize = m_packet_data[1];
            if ( get_protocol().is_crypt() )
            {
                // Wenn die Daten verschluesselt werden, dann kommt es zu einer Datenmengenerhoehung
                // aufgrund der 16 Byte Bloecke beim AES.
                // Zusaetzlich werden die Daten um einen CRC und Anzahl Nutzdaten erweitert, daher (count + 5)!
                uint32_t count_blocks = (statusSize + 5) / 16;
                if ((statusSize + 5) % 16)
                {
                    count_blocks++;
                }

                statusSize = count_blocks * 16;
            }

            // Paketframe noch dazurechnen
            statusSize += 10;

            // Aktuell werden folgende Werte im Status geliefert:
            // - 32bit bereits verarbeitete Anzahl Byte
            // - 32bit crc zu dieser Verarbeitungsposition
            // Speicher fuer Status-Meldung.
            boost::scoped_array<uint8_t> statusReceive(new uint8_t[statusSize]);
            uint8_t *statusReceivePos;

            boost::scoped_array<uint8_t> status(new uint8_t[statusSize]);
            uint8_t *statusPos;

            // In einer Schleife so lange die Statusmeldungen abrufen,
            // bis die Information vorliegt, dass alle Daten verarbeitet wurden.
            do
            {
                receiveCount = statusSize;
                receivedCount = 0;
                masked = false;
                // Annahme beginnt am Anfang des Puffers.
                statusPos = status.get();

                // Es wird maximal 25 Sekunden zwischen den einzelnen Statusmeldungen gewartet.
                int retries = IV_PENDING_TRIALS( get_connection().get_read_timeout() );
                for ( ; retries >= 0; retries-- )
                {
                    // Leseschleife um Maskierte Zeichen herausfiltern.
                    while ( receiveCount && (res = read_data( statusReceive.get(), receiveCount, receivedCount, 1 )) == IV_ERROR_SUCCESS )
                    {
                        // Da nun Zeichen empfangen wurden wird nicht weiter auf den Lesebeginn gewartet.
                        retries = 0;

                        // Fuer jedes maskierte Zeichen muss nachher eins nachgelesen werden.
                        receiveCount = 0;

                        // Auswertung beginnt am Anfang des Temp Puffers.
                        statusReceivePos = statusReceive.get();

                        // Gelesene Daten in Status uebernehmen
                        for ( ; receivedCount; receivedCount-- )
                        {
                            // Aktuelles Zeichen "maskiert" ?
                            if ( masked )
                            {
                                switch ( *statusReceivePos )
                                {
                                case 0x00:
                                    *statusPos = 0x7F;
                                    break;

                                case 0x01:
                                    *statusPos = 0x7E;

                                    break;
                                default:
                                    // Zeichen bleibt unveraendert.
                                    *statusPos = *statusReceivePos;
                                    break;
                                }

                                // Flag fuer "maskiert" zurucknehmen.
                                masked = false;

                                // Naechstes Zeichen
                                statusPos++;
                            }
                            else if ( *statusReceivePos == 0x7F )
                            {
                                // Flag fuer "maskiert" setzen. In der naechsten Runde mit seiner Entsprechung ersetzen.
                                masked = true;
                                // Fuer dieses "maskiert" ist ein nachlesen erforderlich.
                                receiveCount++;
                            }
                            else
                            {
                                // Unmaskiertes Zeichen uebernehmen.
                                *statusPos = *statusReceivePos;
                                // Naechstes Zeichen
                                statusPos++;
                            }

                            // Pufferueberlauf vermeiden.
                            if ( (statusPos - status.get()) > statusSize )
                            {
                                break;
                            }

                            // Auf naechstes Zeichen fuer Uebernahme wechseln.
                            statusReceivePos++;
                        }
                    }
                }

                // Keine Statusmeldung im vorgegeben Zeitraum?
                if ( retries == 0 )
                {
                    trace( "%s(%i):ERROR(1): %s [ status timeout ]\n", __FILE__, __LINE__, __func__ );
                    return -IV_ERROR_NO_REQUEST;                      
                }

                if ( res != IV_ERROR_SUCCESS )
                {
                    trace( "%s(%i):ERROR(1): %s [ ReceiveData fail %u ]\n", __FILE__, __LINE__, __func__, res );
                    return res;                      
                }

                // Wurde die erwartete Anzahl Zeichen empfangen?
                if ( (statusPos - status.get()) != statusSize )
                {
                    trace( "%s(%i):ERROR(1): %s [ status size %u < %u ]\n", __FILE__, __LINE__, __func__, (statusPos - status.get()), statusSize );
                    return -IV_ERROR_UNREACHABLE;                      
                }

                // CRC von Kopfdaten pruefen.
                if ( GetCRC16( status.get(), 6 ) != *(uint16_t *)(status.get() + 6) )
                {
                    trace( "%s(%i):ERROR(1): %s [ head crc 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, GetCRC16( status.get(), 6 ), *(uint16_t *)(status.get() + 6) );
                    return -IV_ERROR_CRC_HEAD;
                }
                // Laengenabgabe pruefen.
                if ( *(uint32_t *)status.get() != (statusSize - 8) )
                {
                    trace( "%s(%i):ERROR(1): %s [ length 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, *(uint32_t *)status.get(), (statusSize - 8) );
                    return -IV_ERROR_RECEIVE;
                }
                // Gesamt-CRC pruefen
                if ( GetCRC16( status.get() + 8, statusSize - 10) != *(uint16_t *)(status.get() + statusSize - 2) )
                {
                    trace( "%s(%i):ERROR(1): %s [ paket crc 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, GetCRC16( status.get() + 8, statusSize - 10 ), *(uint16_t *)(status.get() + statusSize - 2) );
                    return -IV_ERROR_CRC_PACKET;
                }
                uint8_t *lBuf = status.get() + 8;
                std::string _status( (char *)lBuf, statusSize - 10);
                // Verschluesselung aktiv?
                if ( get_protocol().is_crypt() )
                {
                    // Datenteil entschluesseln.
                    if ( (res = get_protocol().read_stream_crypt( _status )) != IV_ERROR_SUCCESS )
                    {
                        trace( "%s(%i):ERROR(1): %s [ decrypt fail ]\n", __FILE__, __LINE__, __func__ );
                        return res;
                    }
                }
                // Daten im Puffer neu ausrichten.
                lBuf = status.get();
                for ( std::size_t i = 0; i < _status.size() && i < (statusSize - 10); i++ )
                {
                    lBuf[i] = (uint8_t)_status[i];
                }

                uint32_t s;
                uint32_t c;
                s = *(uint32_t *)status.get();
                if ( s <= count )
                {
                    c = *(uint32_t *)&status[sizeof( uint32_t )];
                    crcChunk = s - uint32_t( crcPos - (char *)buffer );
                    crc = get_crc32( (uint8_t *)crcPos, crcChunk, crc, s == count );
                    crcPos += crcChunk;
                    trace( "%s(%i):INFO(1): %s [ %3u: progression: %u / %u ( %u )\tcrc check: %s ]\n", __FILE__, __LINE__, __func__, statusCount++, s, count, count - s, (crc == c) ? "OK" : "FAIL" );

                    if ( crc != c )
                    {
                        trace( "%s(%i):ERROR(1): %s [ status progression crc 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, c, crc );
                        return -IV_ERROR_CRC_SEGMENT;
                    }
                }
                else
                {
                    trace( "%s(%i):ERROR(1): %s [ status progression length %u > %u ]\n", __FILE__, __LINE__, __func__, s, count );
                    return -IV_ERROR_CRC_SEGMENT;
                }

                // remoteCall-Back Funktion aufrufen, ggf. abbruch.
                if ( get_protocol().m_call_back_function != nullptr ) 
                {
                    if ( !get_protocol().m_call_back_function() ) 
                    {
                        trace( "%s(%i):ERROR(1): %s [ CallBack abbruch ]\n", __FILE__, __LINE__, __func__ );
                        // Reservierten Speicher freigeben.
                        return -IV_ERROR_BREAK;
                    }
                }

                if ( m_call_back_function_with_parameters != nullptr )
                {
                    if ( !m_call_back_function_with_parameters( (uint32_t)count, (uint32_t)s ) )
                    {
                        result = -IV_ERROR_BREAK;
                        trace_error( device_address, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM, result );
                    }
                }

            } while ( *(uint32_t *)status.get() < count && res == IV_ERROR_SUCCESS );
        }
        else
        {
            // Zu wendige Antwortdaten
            result = -IV_ERROR_RECEIVE;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_stream(uint8_t device_address, Stream_type type, char *buffer, std::size_t count)
{
    trace( "%s(%i):INFO(3): %s [ %u, %i, %p, %u ]\n", __FILE__, __LINE__, __func__, device_address, type, buffer, count );

    int result = IV_ERROR_SUCCESS;

    /*
    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != device_address)
    {
    // Umgebung zuruecksetzen
    reset_error();

    // Remoteadresse setzen.
    setDeviceIdRemote(device_address);

    // Gesamten Datenblock aufteilen
    for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
    {
    // Datengroesze 
    sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

    // Meberpuffer aufbereiten
    *(uint32_t*)_packetData = address + addressOffset;
    // Angeforderte Laenge eintragen
    *(uint16_t*)&_packetData[sizeof(uint32_t)] = sizeData;

    // Variable aufbereiten
    sizeData = sizeof(uint32_t) + sizeof(uint16_t);

    if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_RECEIVE_STREAM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
    {
    // Fehler ausgeben
    traceError(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH);
    return _result;
    }

    // OK

    // Empfangene Daten uebernehmen
    for (i=0; i < (sizeData - (signed)sizeof(uint32_t)); i++)
    {				
    buf[addressOffset + i] = _packetData[sizeof(uint32_t) + i];
    }	
    }
    }
    else
    {
    uint8_t tmp[4];
    count = 0;

    // Antwort des Befehls senden.
    if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_RECEIVE_STREAM, tmp, count)) != IV_ERROR_SUCCESS)
    {			
    Trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
    return _result;
    }
    }
    */
    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_system_key(uint8_t device_address, char* key)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, key);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Puffer loeschen
        memset(m_packet_data, 0, sizeof(m_packet_data));
        // Der Passwortpuffer wird mit 17 Zeichen verschluesselt
        count = 17;

        // Initialisieren der Verschluesselung
        get_protocol().df4_load_crypt_key();
        get_protocol().df4_init(TRUE);

        // Nur wenn Passwort angegeben wurde.
        if (strlen(key)) {

            // Passwort uebernehmen
            strncpy((char*)m_packet_data, key, 16);
        }

        // Verschluesseln
        get_protocol().df4(m_packet_data, count);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY_CHECK, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY_CHECK, result);

            // Fehler auswerten
            if (((result >> 8) & 0xFF) == 2)
            {
                // Funktion unbekannt
            }
            else if ((result & 0xFF) == 1)
            {
                // Funktionsfehler, passwort falsch
            }

            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY_CHECK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_system_key(uint8_t device_address, char* key)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, device_address, key);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Keine Daten zu senden
        count = 0;

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY, result);
            return result;
        }

        // n Antwortdaten erwartet
        if (count != 17)
        {
            trace("%s(%i):WARNING(1): %s [ Laenge des Passwortschlüssels mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Wert uebernehmen
        strcpy(key, (char*)m_packet_data);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::enter_transparency_mode( uint8_t device_address, uint16_t id, uint16_t flags, uint32_t read_buffer_size, uint32_t write_buffer_size, int timeout )
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %p, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, id, flags, read_buffer_size, write_buffer_size, timeout);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        *reinterpret_cast<uint16_t *>( m_packet_data ) = id;
        count = sizeof(uint16_t);
        *reinterpret_cast<uint16_t *>( &m_packet_data[count] ) = flags;
        count += sizeof(uint16_t);
        *reinterpret_cast<uint32_t *>( &m_packet_data[count] ) = read_buffer_size;
        count += sizeof(uint32_t);
        *reinterpret_cast<uint32_t *>( &m_packet_data[count] ) = write_buffer_size;
        count += sizeof(uint32_t);
        *reinterpret_cast<int *>( &m_packet_data[count] ) = timeout;
        count += sizeof(int);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE, result);
            return result;
        }

        // Den nun genutzten Mode halten.
        if ( (flags & 0x01) == 0x01 )
            m_transparency_mode = tm_raw;
        else
            m_transparency_mode = tm_masked;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::exit_transparency_mode( uint8_t device_address )
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, device_address );

    int result = IV_ERROR_SUCCESS;
    std::size_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        char exitCode[] = { '~', '~', '~', '\0' };
        if ( (result = get_connection().write_data( (const uint8_t *)&exitCode[0], sizeof( exitCode ), count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE, result);
            return result;
        }

        // Den nun genutzten Mode halten.
        m_transparency_mode = tm_off;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
bool Datafox::Communication::Module_system::is_transparency_mode()
{
    return m_transparency_mode != tm_off;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::write_led_state( uint8_t device_address, uint8_t slot, uint8_t id, uint8_t state )
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, slot, id, state);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Den anzusprechenden Slot uebernehmen.
        m_packet_data[count] = slot;
        count += sizeof(uint8_t);
        // Die anzusprechende LED uebernehmen.
        m_packet_data[count] = id;
        count += sizeof(uint8_t);
        // Den gewünschten Zustand uebernehmen.
        m_packet_data[count] = state;
        count += sizeof(uint8_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_LED, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_SET_LED, result);
            return result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_LED, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_system::read_led_state(uint8_t device_address, uint8_t slot, uint8_t id, uint8_t &state)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, device_address, slot, id, state);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver(device_address);

        // Den anzusprechenden Slot uebernehmen.
        m_packet_data[count] = slot;
        count += sizeof(uint8_t);
        // Die anzusprechende LED uebernehmen.
        m_packet_data[count] = id;
        count += sizeof(uint8_t);

        if ((result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_LED, m_packet_data, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            trace_error(device_address, IV_MODULE_SYSTEM, IV_FUNCTION_GET_LED, result);
            return result;
        }

        // OK
        if (count != 3 * sizeof(uint8_t))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        state = (m_packet_data[2] != 0xFF) ? m_packet_data[2] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_LED, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

#if !defined( DATAFOX_STUDIO_IV ) && !defined( DF_API_PRODUCTION )
// Funktions-Stumps

// MODULE_HARDWARE
int Datafox::Communication::Module_system::write_display_type( uint8_t device_address, uint8_t val, uint16_t value2, bool checked )
{ 
    (void)val;
    (void)value2;
    (void)checked;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Datafox::Communication::Module_system::write_keypad_type( uint8_t device_address, uint8_t val, uint16_t value2, bool checked )
{
    (void)val;
    (void)value2;
    (void)checked;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Datafox::Communication::Module_system::write_transponder_reader_type( uint8_t device_address, uint8_t val, uint16_t value2, bool checked )
{
    (void)val;
    (void)value2;
    (void)checked;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Datafox::Communication::Module_system::write_serial_number( uint8_t device_address, uint32_t serial, uint32_t value2 )
{
    (void)serial;
    (void)value2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

#endif
